<?php
/**
 * ***** BEGIN LICENSE BLOCK *****
 * This file is part of QPoll.
 * Uses parts of code from Quiz extension (c) 2007 Louis-Rémi BABE. All rights reserved.
 *
 * QPoll is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * QPoll is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with QPoll; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ***** END LICENSE BLOCK *****
 *
 * QPoll is a poll tool for MediaWiki.
 * 
 * To activate this extension :
 * * Create a new directory named QPoll into the directory "extensions" of MediaWiki.
 * * Place the files from the extension archive there.
 * * Add this line at the end of your LocalSettings.php file :
 * require_once "$IP/extensions/QPoll/qp_user.php";
 * 
 * @version 0.3.0
 * @link http://www.mediawiki.org/wiki/Extension:QPoll
 * @author QuestPC <questpc@rambler.ru>
 */
if ( !defined( 'MEDIAWIKI' ) ) {
	die( "This file is part of the QPoll extension. It is not a valid entry point.\n" );
}

class PollResults extends SpecialPage {

	public function __construct() {
		parent::__construct( 'PollResults', 'delete' );
		wfLoadExtensionMessages('QPoll');
	}

	static $skin = NULL;
	static $UsersLink = "";
	static $PollsLink = "";

	public function execute( $par ) {
		global $wgOut, $wgRequest;
		global $wgServer; // "http://www.yourserver.org"
							// (should be equal to 'http://'.$_SERVER['SERVER_NAME'])
		global $wgScript;   // "/subdirectory/of/wiki/index.php"
		global $wgUser;
		global $qp_ExtDir, $qp_ScriptPath;
		if ( !$wgUser->isAllowed( 'delete' ) ) {
			$wgOut->permissionRequired('delete');
			return;
		}
		$qp_sql_tables = array(
			"qp_poll_desc",
			"qp_question_desc",
			"qp_question_categories",
			"qp_question_proposals",
			"qp_question_answers",
			"qp_users");
		$wgOut->addExtensionStyle( $qp_ScriptPath . '/qp_results.css' );
		if ( self::$skin == NULL ) {
			self::$skin = $wgUser->getSkin();
		}
		if ( self::$UsersLink == "" ) {
			self::$UsersLink = self::$skin->link( $this->getTitle(), wfMsg( 'qp_users_list' ), array( "style"=>"font-weight:bold;" ), array( 'action'=>'users' ) );
		}
		if ( self::$PollsLink == "" ) {
			self::$PollsLink = self::$skin->link( $this->getTitle(), wfMsg( 'qp_polls_list' ), array( "style"=>"font-weight:bold;" ) );
		}
		$wgOut->addHTML( '<div class="qpoll">' );
		$output = "";
		$this->setHeaders();
		$db = & wfGetDB( DB_SLAVE );
		// check whether the tables were initialized
		$tablesFound = 0;
		foreach ( $qp_sql_tables as $table ) {
			$tname = str_replace( "`", "'", $db->tableName( $table ) );
			$res = $db->query( "SHOW TABLE STATUS LIKE $tname" );
			if ( $db->numRows( $res ) > 0 ) {
				$tablesFound++;
			}
		}
		if ( $tablesFound  == 0 ) {
			# no tables were found, initialize the DB completely
			$result = $db->sourceFile( $qp_ExtDir . "/qpoll.src" );
			if ( $result !== true ) {
				$wgOut->addHTML( $result );
			} else {
				$wgOut->addHTML( "Tables were initialized.<br />Please reload this page to view future polls results." );
			}
		} else {
			if ( $tablesFound != count( $qp_sql_tables ) ) {
				# some tables are missing, serious DB error
				$wgOut->addHTML( "Some of the extension database tables are missing.<br />Please restore from backup or drop all qp_ prefix tables, then reload this page." );
		  } else	{
				# normal processing
				$cmd = $wgRequest->getVal( 'action' );
				if ( $cmd === NULL ) {
					list( $limit, $offset ) = wfCheckLimits();
					$qpl = new qp_PollsList();
					$qpl->doQuery( $offset, $limit );
				} else {
					$pid = $wgRequest->getVal( 'id' );
					$uid = $wgRequest->getVal( 'uid' );
					$question_id = $wgRequest->getVal( 'qid' );
					$proposal_id = $wgRequest->getVal( 'pid' );
					$cid = $wgRequest->getVal( 'cid' );
					switch ($cmd) {
						case 'stats':
							if ( $pid !==NULL ) {
								$pid = intval( $pid );
								$output = self::getPollsLink();
								$output .= self::getUsersLink();
								$output .= $this->showVotes( $pid );
							}
							break;
						case 'stats_xls':
								if ( $pid !==NULL ) {
									$pid = intval( $pid );
									$this->votesToXLS( $pid );
								}
							break;
						case 'uvote':
							if ( $pid !== NULL && $uid !==NULL ) {
								$pid = intval( $pid );
								$uid = intval( $uid );
								$output = self::getPollsLink();
								$output .= self::getUsersLink();
								$output .= $this->showUserVote( $pid, $uid );
							}
							break;
						case 'qpcusers':
							if ( $pid !==NULL && $question_id !==NULL && $proposal_id !== NULL && $cid !== NULL ) {
								$pid = intval( $pid );
								$question_id = intval( $question_id );
								$proposal_id = intval( $proposal_id );
								$cid = intval( $cid );
								list( $limit, $offset ) = wfCheckLimits();
								$qucl = new qp_UserCellList( $cmd, $pid, $question_id, $proposal_id, $cid );
								$qucl->doQuery( $offset, $limit );
							}
							break;
						case 'users':
							list( $limit, $offset ) = wfCheckLimits();
							$qul = new qp_UsersList( $cmd, $pid );
							$qul->doQuery( $offset, $limit );
							break;
						case 'upolls':
						case 'nupolls':
							if ( $uid !== NULL ) {
								$uid = intval( $uid );
								list( $limit, $offset ) = wfCheckLimits();
								$qupl = new qp_UserPollsList( $cmd, $uid );
								$qupl->doQuery( $offset, $limit );
							}
							break;
						case 'pulist':
						case 'npulist':
							if ( $pid !== NULL ) {
								$pid = intval( $pid );
								list( $limit, $offset ) = wfCheckLimits();
								$qpul = new qp_PollUsersList( $cmd, $pid );
								$qpul->doQuery( $offset, $limit );
							}
							break;
					}
				}
			}
		}
		$wgOut->addHTML( $output . '</div>' );
	}

	private function showUserVote( $pid, $uid ) {
		$output = "";
		if ( $pid !== NULL && $uid !== NULL ) {
			$pollStore = new qp_PollStore( array( 'from'=>'pid', 'pid'=> $pid ) );
			if ( $pollStore->pid !== NULL ) {
				$pollStore->loadQuestions();
				$userName = $pollStore->getUserName( $uid );
				if ( $userName !== false ) {
					$userTitle = Title::makeTitleSafe( NS_USER, $userName );
					$user_link = self::$skin->link( $userTitle, $userName );
					$pollStore->setLastUser( $userName );
					if ( $pollStore->loadUserVote() ) {
						$poll_title = $pollStore->getTitle();
						$output .= wfMsg( 'qp_browse_to', $user_link . "<br />\n" );
						$output .= wfMsg( 'qp_browse_to', self::$skin->link( $poll_title, $poll_title->getPrefixedText() . ' (' .		$pollStore->mPollId . ")<br />\n" ) );
						foreach ( $pollStore->Questions as $qkey => &$qdata ) {
							$output .= "<br />\n<b>" . $qkey . ".</b> " . $qdata->CommonQuestion . "<br />\n";
							$output .= $this->displayUserQuestionVote( $qdata );
						}
					}
				}
			}
		}
		return $output;
	}

	private function displayUserQuestionVote( &$qdata ) {
		$output = "<div class=\"qpoll\">\n" . "<table class=\"pollresults\">\n";
		$output .= qp_Renderer::displayRow( $qdata->CategorySpans, array( 'class'=>'spans' ), 'th', array( 'count'=>'colspan', 'name'=>0 ) );
		$output .= qp_Renderer::displayRow( $qdata->Categories, '', 'th', array( 'name'=>0 ) );
		# multiple choice polls doesn't use real spans, instead, every column is like "span"
		$spansUsed = count( $qdata->CategorySpans ) > 0 || $qdata->type == "multipleChoicePoll";
		foreach( $qdata->ProposalText as $propkey => &$proposal_text ) {
			$row = Array();
			foreach ( $qdata->Categories as $catkey => &$cat_name ) {
				$cell = Array( 0=>"" );
				if ( array_key_exists( $propkey, $qdata->ProposalCategoryId ) &&
							in_array( $catkey, $qdata->ProposalCategoryId[ $propkey ] ) ) {
					$cell[ 0 ] = "+";
				}
				if ( $spansUsed ) {
					if ( $qdata->type == "multipleChoicePoll" ) {
						$cell[ "class" ] = ( ( $catkey & 1 ) === 0 ) ? "spaneven" : "spanodd";
					} else {
						$cell[ "class" ] = ( ( $qdata->Categories[ $catkey ][ "spanId" ] & 1 ) === 0 ) ? "spaneven" : "spanodd";
					}
				} else {
					$cell[ "class" ] = "stats";
				}
				$row[] = $cell;
			}
			$row[] = array( 0=>$proposal_text, "style"=>"text-align:left;" );
			$output .= qp_Renderer::displayRow( $row );
		}
		$output .= "</table>\n" . "</div>\n";
		return $output;
	}

	private function showVotes( $pid ) {
		$output = "";
		if ( $pid !== NULL ) {
			$pollStore = new qp_PollStore( array( 'from'=>'pid', 'pid'=> $pid ) );
			if ( $pollStore->pid !== NULL ) {
				$pollStore->loadQuestions();
				$pollStore->loadAllVotes();
				$pollStore->calculateStatistics();
				$poll_title = $pollStore->getTitle();
				$output .= wfMsg( 'qp_browse_to', self::$skin->link( $poll_title, $poll_title->getPrefixedText() . ' (' . $pollStore->mPollId . ")<br />\n" ) );
				$output .= self::$skin->link( $this->getTitle(), wfMsg( 'qp_export_to_xls' ), array( "style"=>"font-weight:bold;" ), array( 'action'=>'stats_xls', 'id'=>$pid ) );
				foreach ( $pollStore->Questions as $qkey => &$qdata ) {
					$output .= "<br />\n<b>" . $qkey . ".</b> " . $qdata->CommonQuestion . "<br />\n";
					$output .= $this->displayQuestionStats( $pid, $qdata );
				}
			}
		}
		return $output;
	}

	private function votesToXLS( $pid ) {
		global $qp_ExtDir;
		$output = "";
		if ( $pid !== NULL ) {
			$pollStore = new qp_PollStore( array( 'from'=>'pid', 'pid'=> $pid ) );
			if ( $pollStore->pid !== NULL ) {
				$poll_id = $pollStore->getPollId();
				$pollStore->loadQuestions();
				$pollStore->loadAllVotes();
				$pollStore->calculateStatistics();
				try {
					require_once( $qp_ExtDir . '/Excel/Excel_Writer.php' );
					$xls_fname = tempnam( "", ".xls" );
					$xls_workbook = &new Spreadsheet_Excel_Writer_Workbook( $xls_fname );
					$xls_workbook->setVersion( 8 );
					$xls_worksheet = &$xls_workbook->addworksheet();
					$xls_worksheet->setInputEncoding( "utf-8" );
					$xls_worksheet->setPaper( 9 );
					$xls_rownum = 0;
					$percent_num_format = '[Blue]0.0%;[Red]-0.0%;[Black]0%';
					$format_heading = &$xls_workbook->addformat( array( 'bold'=>1 ) );
					$format_percent = &$xls_workbook->addformat( array( 'fgcolor'=>0x1A, 'border'=>1 ) );
					$format_percent->setAlign('left');
					$format_percent->setNumFormat( $percent_num_format );
					$format_even = &$xls_workbook->addformat( array( 'fgcolor'=>0x2A, 'border'=>1 ) );
					$format_even->setAlign('left');
					$format_even->setNumFormat( $percent_num_format );
					$format_odd = &$xls_workbook->addformat( array( 'fgcolor'=>0x23, 'border'=>1 ) );
					$format_odd->setAlign('left');
					$format_odd->setNumFormat( $percent_num_format );
					$first_question = true;
					foreach ( $pollStore->Questions as $qkey => &$qdata ) {
						if ( $first_question ) {
							$totalUsersAnsweredQuestion = $pollStore->totalUsersAnsweredQuestion( $qdata );
							$xls_worksheet->write( $xls_rownum, 0, $totalUsersAnsweredQuestion, $format_heading );
							$xls_worksheet->write( $xls_rownum++, 1, wfMsg( 'qp_users_answered_questions', $totalUsersAnsweredQuestion ), $format_heading );
							$xls_rownum++;
							$first_question = false;
						}
						$xls_worksheet->write( $xls_rownum, 0, $qdata->question_id, $format_heading );
						$xls_worksheet->write( $xls_rownum++, 1, qp_Excel::prepareExcelString( $qdata->CommonQuestion ), $format_heading );
						if ( count( $qdata->CategorySpans ) > 0 ) {
							$row = array();
							foreach( $qdata->CategorySpans as &$span ) {
								$row[] = qp_Excel::prepareExcelString( $span[ "name" ] );
								for ( $i = 1; $i < $span[ "count" ]; $i++ ) {
									$row[] = "";
								}
							}
							$xls_worksheet->writerow( $xls_rownum++, 0, $row );
						}
						$row = array();
						foreach( $qdata->Categories as &$categ ) {
							$row[] = qp_Excel::prepareExcelString( $categ[ "name" ] );
						}
						$xls_worksheet->writerow( $xls_rownum++, 0, $row );
/*
						foreach( $qdata->Percents as $pkey=>&$percent ) {
							$xls_worksheet->writerow( $xls_rownum + $pkey, 0, $percent );
						}
*/
						$percentsTable = Array();
						$spansUsed = count( $qdata->CategorySpans ) > 0 || $qdata->type == "multipleChoicePoll";
						foreach( $qdata->ProposalText as $propkey => &$proposal_text ) {
							if ( isset( $qdata->Percents[ $propkey ] ) ) {
								$row = $qdata->Percents[ $propkey ];
								foreach ( $row as $catkey => &$cell ) {
									$cell = array( 0=>$cell );
									if ( $spansUsed ) {
										if ( $qdata->type == "multipleChoicePoll" ) {
											$cell[ "format" ] = ( ( $catkey & 1 ) === 0 ) ? $format_even : $format_odd;
										} else {
											$cell[ "format" ] = ( ( $qdata->Categories[ $catkey ][ "spanId" ] & 1 ) === 0 ) ? $format_even : $format_odd;
										}
									}
								}
							} else {
								$row = array_fill( 0, count( $qdata->Categories ), '' );
							}
							$percentsTable[] = $row;
						}
						qp_Excel::writeFormattedTable( $xls_worksheet, $xls_rownum, 0, $percentsTable, $format_percent );
						$row = array();
						foreach ( $qdata->ProposalText as $ptext ) {
							$row[] = qp_Excel::prepareExcelString( $ptext );
						}
						$xls_worksheet->writecol( $xls_rownum, count( $qdata->Categories ), $row );
						$xls_rownum += count( $qdata->ProposalText ) + 1;
					}
					$xls_workbook->close();
					header( 'Content-Type: application/x-msexcel; name="' . $poll_id . '.xls"' );
					header( 'Content-Disposition: inline; filename="' . $poll_id . '.xls"' );
					@$fxls=fopen( $xls_fname, "rb" );
					@fpassthru( $fxls );
					@unlink( $xls_fname );
					exit();
				} catch( Exception $e ) {
					die( "Error while exporting poll statistics to Excel table\n" );
				}
			}
		}
	}

	private function displayQuestionStats( $pid, &$qdata ) {
		$current_title = $this->getTitle();
		$output = "<div class=\"qpoll\">\n" . "<table class=\"pollresults\">\n";
		$output .= qp_Renderer::displayRow( $qdata->CategorySpans, array( 'class'=>'spans' ), 'th', array( 'count'=>'colspan', 'name'=>0 ) );
		$output .= qp_Renderer::displayRow( $qdata->Categories, '', 'th', array( 'name'=>0 ) );
		# multiple choice polls doesn't use real spans, instead, every column is like "span"
		$spansUsed = count( $qdata->CategorySpans ) > 0 || $qdata->type == "multipleChoicePoll";
		foreach( $qdata->ProposalText as $propkey => &$proposal_text ) {
			if ( isset( $qdata->Votes[ $propkey ] ) ) {
				if ( $qdata->Percents === NULL ) {
					$row = $qdata->Votes[ $propkey ];
				} else {
					$row = $qdata->Percents[ $propkey ];
					foreach ( $row as $catkey => &$cell ) {
						$formatted_cell = str_replace( " ", "&ensp;", sprintf( '%3d%%', intval( round( 100 * $cell ) ) ) );
						# only percents !=0 are displayed as link
						if ( $cell == 0.0 && $qdata->question_id !==NULL ) {
							$cell = array( 0=> $formatted_cell, "style"=>"color:gray" );
						} else {
							$cell = array( 0=>self::$skin->link( $current_title, $formatted_cell,
								array( "title"=>wfMsg( 'qp_votes_count', $qdata->Votes[ $propkey ][ $catkey ] ) ),
								array( "action"=>"qpcusers", "id"=>$pid, "qid"=>$qdata->question_id, "pid"=>$propkey, "cid"=>$catkey ) ) );
						}
						if ( $spansUsed ) {
							if ( $qdata->type == "multipleChoicePoll" ) {
								$cell[ "class" ] = ( ( $catkey & 1 ) === 0 ) ? "spaneven" : "spanodd";
							} else {
								$cell[ "class" ] = ( ( $qdata->Categories[ $catkey ][ "spanId" ] & 1 ) === 0 ) ? "spaneven" : "spanodd";
							}
						} else {
							$cell[ "class" ] = "stats";
						}
					}
				}
			} else {
				# this proposal has no statistics (no votes)
				$row = array_fill( 0, count( $qdata->Categories ), '' );
			}
			$row[] = array( 0=>$proposal_text, "style"=>"text-align:left;" );
			$output .= qp_Renderer::displayRow( $row );
		}
		$output .= "</table>\n" . "</div>\n";
		return $output;
	}

	static function getUsersLink() {
		return self::$UsersLink . "<br />\n";
	}

	static function getPollsLink() {
		return self::$PollsLink . "<br />\n";
	}

}

abstract class qp_QueryPage extends QueryPage {

	static $skin = null;

	public function __construct() {
		global $wgUser;
		if ( self::$skin == NULL ) {
			self::$skin = $wgUser->getSkin();
		}
	}

	function doQuery( $offset, $limit, $shownavigation=true ) {
		global $wgUser, $wgOut, $wgLang, $wgContLang;

		$res = $this->getIntervalResults( $offset, $limit );
		$num = count($res);

		$sk = $wgUser->getSkin();
		$sname = $this->getName();

		if($shownavigation) {
			$wgOut->addHTML( $this->getPageHeader() );

			// if list is empty, display a warning
			if( $num == 0 ) {
				$wgOut->addHTML( '<p>' . wfMsgHTML('specialpage-empty') . '</p>' );
				return;
			}

			$top = wfShowingResults( $offset, $num );
			$wgOut->addHTML( "<p>{$top}\n" );

			// often disable 'next' link when we reach the end
			$atend = $num < $limit;

			$sl = wfViewPrevNext( $offset, $limit ,
				$wgContLang->specialPage( $sname ),
				wfArrayToCGI( $this->linkParameters() ), $atend );
			$wgOut->addHTML( "<br />{$sl}</p>\n" );
		}
		if ( $num > 0 ) {
			$s = array();
			if ( ! $this->listoutput )
				$s[] = $this->openList( $offset );

			foreach ($res as $r) {
				$format = $this->formatResult( $sk, $r );
				if ( $format ) {
					$s[] = $this->listoutput ? $format : "<li>{$format}</li>\n";
				}
			}

			if ( ! $this->listoutput )
				$s[] = $this->closeList();
			$str = $this->listoutput ? $wgContLang->listToText( $s ) : implode( '', $s );
			$wgOut->addHTML( $str );
		}
		if($shownavigation) {
			$wgOut->addHTML( "<p>{$sl}</p>\n" );
		}
		return $num;
	}

	function getName() {
		return "PollResults";
	}

	function isExpensive() {
		return false; // disables caching
	}

	function isSyndicated() {
		return false;
	}

}

/* list of all users */
class qp_UsersList extends qp_QueryPage {
	function getIntervalResults( $offset, $limit ) {
		$result = array();
		$db = & wfGetDB( DB_SLAVE );
		$res = $db->select( 'qp_users',
			array( 'uid', 'name as username' ),
			'',
			__METHOD__,
			array( 'ORDER BY'=>'name',
						'OFFSET'=>intval( $offset ),
						'LIMIT'=>intval( $limit ) ) );
		while( $row = $db->fetchObject( $res ) ) {
			$result[] = $row;
		}
		return $result;
	}

	function formatResult( $skin, $result ) {
		global $wgLang, $wgContLang;
		$link = "";
		if ( $result !== NULL ) {
			$uid = intval( $result->uid );
			$userName = $result->username;
			$userTitle = Title::makeTitleSafe( NS_USER, $userName );
			$user_link = self::$skin->link( $userTitle, $userName );
			$user_polls_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_user_polls_link' ) , array(), array( "uid"=>$uid, "action"=>"upolls" ) );
			$user_missing_polls_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_user_missing_polls_link' ) , array(), array( "uid"=>$uid, "action"=>"nupolls" ) );
			$link = $user_link . ': ' . $user_polls_link . ', ' . $user_missing_polls_link;
		}
		return $link;
	}

	function linkParameters() {
		$params[ 'action' ] = 'users';
		return $params;
	}

	function getPageHeader() {
		return PollResults::getPollsLink() . '<span class="head">' . wfMsg( 'qp_users_list' ) . '</span>';
	}

}

/* list of polls in which selected user (did not|participated) */
class qp_UserPollsList extends qp_QueryPage {
	var $uid;
	var $inverse;
	var $cmd;

	public function __construct( $cmd, $uid ) {
		parent::__construct();
		$this->uid = intval( $uid );
		$this->cmd = $cmd;
		$this->inverse = ( $cmd == "nupolls" );
	}

	function getPageHeader() {
		global $wgLang, $wgContLang;
		# fake pollStore to get username by uid: avoid to use as much as possible
		$pollStore = new qp_PollStore();
		$userName = $pollStore->getUserName( $this->uid );
		if ( $userName !== false ) {
			$userTitle = Title::makeTitleSafe( NS_USER, $userName );
			$user_link = self::$skin->link( $userTitle, $userName );
			return PollResults::getPollsLink() . PollResults::getUsersLink() . '<span class="head">' . wfMsg( $this->inverse ? 'qp_user_missing_polls_link' : 'qp_user_polls_link' ) . ' ' .  $user_link . '</span>';
		}
	}

	function getIntervalResults( $offset, $limit ) {
		$result = Array();
		$db = & wfGetDB( DB_SLAVE );
		$page = $db->tableName( 'page' );
		$qp_poll_desc = $db->tableName( 'qp_poll_desc' );
		$qp_question_answers = $db->tableName( 'qp_question_answers' );
		$query = "SELECT pid, page_namespace AS ns, page_title AS title, poll_id, fragment_id ";
		$query .= "FROM ($qp_poll_desc, $page) ";
		$query .= " WHERE page_id=article_id AND pid " . ( $this->inverse ? "NOT " : "" ) . "IN ";
		$query .= "(SELECT DISTINCT pid ";
		$query .= "FROM $qp_question_answers ";
		$query .= "WHERE uid=" . $db->addQuotes( $this->uid ) . ") ";
		$query .= "ORDER BY page_namespace, page_title, poll_id ";
		$query .= "LIMIT " . intval( $offset ) . ", " . intval( $limit );
		$res = $db->query( $query, __METHOD__ );
		while( $row = $db->fetchObject( $res ) ) {
			$result[] = $row;
		}
		return $result;
	}

	function formatResult( $skin, $result ) {
		global $wgLang, $wgContLang;
		$poll_title = Title::makeTitle( $result->ns, $result->title, qp_Poll::getPollTitleFragment( $result->fragment_id, '' ) );
		$pagename = htmlspecialchars( $wgContLang->convert( $poll_title->getPrefixedText() ) );
		$pollname = htmlspecialchars( $result->poll_id );
		$goto_link = self::$skin->link( $poll_title, wfMsg( 'qp_source_link' ) );
		$voice_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_voice_link' ) . ( $this->inverse ? "?" : "" ), array(), array( "id"=>intval( $result->pid), "uid"=>$this->uid, "action"=>"uvote" ) );
		$link = wfMsg( 'qp_spec_line', '"' . $pagename . '"', '"' . $pollname . '":' , $voice_link, '', '', '' );
		return $link;
	}

	function linkParameters() {
		$params[ "action" ] = $this->cmd;
		if ( $this->uid !== NULL ) {
			$params[ "uid" ] = $this->uid;
		}
		return $params;
	}

}

/* list of all polls */
class qp_PollsList extends qp_QueryPage {

	function getIntervalResults( $offset, $limit ) {
		$result = array();
		$db = & wfGetDB( DB_SLAVE );
		$res = $db->select(
			array( 'page', 'qp_poll_desc' ),
			array( 'page_namespace as ns', 'page_title as title', 'pid', 'poll_id', 'fragment_id' ),
			'page_id=article_id',
			__METHOD__,
			array( 'ORDER BY'=>'page_namespace, page_title, fragment_id',
						'OFFSET'=>intval( $offset ),
						'LIMIT'=>intval( $limit ) ) );
		while( $row = $db->fetchObject( $res ) ) {
			$result[] = $row;
		}
		return $result;
	}

	function formatResult( $skin, $result ) {
		global $wgLang, $wgContLang;
		$poll_title = Title::makeTitle( $result->ns, $result->title, qp_Poll::getPollTitleFragment( $result->fragment_id, '' ) );
		$pagename = htmlspecialchars( $wgContLang->convert( $poll_title->getPrefixedText() ) );
		$pollname = htmlspecialchars( $result->poll_id );
		$goto_link = self::$skin->link( $poll_title, wfMsg( 'qp_source_link' ) );
		$voices_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_stats_link' ), array(), array( "id"=>intval( $result->pid), "action"=>"stats" ) );
		$users_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_users_link' ), array(), array( "id"=>intval( $result->pid), "action"=>"pulist" ) );
		$not_participated_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_not_participated_link' ), array(), array( "id"=>intval( $result->pid), "action"=>"npulist" ) );
		$link = wfMsg( 'qp_spec_line', '"' . $pagename . '"', '"' . $pollname . '":' , $goto_link .', ', $voices_link . ', ', $users_link . ', ', $not_participated_link );
		return $link;
	}

	function getPageHeader() {
		return PollResults::getUsersLink() . '<span class="head">' . wfMsg( 'qp_polls_list' ) . '</span>';
	}

}

/* list of users, (not|participated) in particular poll, defined by pid */
class qp_PollUsersList extends qp_QueryPage {

	var $pid;
	var $inverse;
	var $cmd;

	public function __construct( $cmd, $pid ) {
		parent::__construct();
		$this->pid = intval( $pid );
		$this->cmd = $cmd;
		$this->inverse = ( $cmd == "npulist" );
	}

	function getPageHeader() {
		global $wgLang, $wgContLang;
		$link = "";
		$db = & wfGetDB( DB_SLAVE );
		$res = $db->select(
			array( 'page', 'qp_poll_desc' ),
			array( 'page_namespace as ns', 'page_title as title', 'poll_id', 'fragment_id' ),
			'page_id=article_id and pid=' . $db->addQuotes( $this->pid ),
			__METHOD__ );
		if ( $row = $db->fetchObject( $res ) ) {
			$poll_title = Title::makeTitle( intval( $row->ns ), $row->title, qp_Poll::getPollTitleFragment( intval( $row->fragment_id ), '' ) );
			$pagename = htmlspecialchars( $wgContLang->convert( $poll_title->getPrefixedText() ) );
			$pollname = htmlspecialchars( $row->poll_id );
			$goto_link = self::$skin->link( $poll_title, wfMsg( 'qp_source_link' ) );
			$spec = wfMsg( 'qp_spec_line', '"' . $pagename . '"', '"' . $pollname . '"', '', '', '', '' );
			$head[] = PollResults::getPollsLink();
			$head[] = PollResults::getUsersLink();
			$head[] = array( '__tag'=>'span', 'class'=>'head', 0=>wfMsg( $this->inverse ? 'qp_not_participated_link' : 'qp_users_link' ) . ' [ '. $spec . ']' );
			$head[] = ' (' . $goto_link . ')';
			$link = qp_Renderer::renderHTMLobject( $head );
		}
		return $link;
	}

	function getIntervalResults( $offset, $limit ) {
		$result = Array();
		$db = & wfGetDB( DB_SLAVE );
		$qp_users = $db->tableName( 'qp_users' );
		$qp_question_answers = $db->tableName( 'qp_question_answers' );
		$query = "SELECT uid, name as username ";
		$query .= "FROM $qp_users ";
		$query .= "WHERE uid " . ( $this->inverse ? "NOT " : "" ) . "IN ";
		$query .= "(SELECT DISTINCT uid FROM $qp_question_answers WHERE pid=" . $db->addQuotes($this->pid) . ") ";
		$query .= "ORDER BY uid ";
		$query .= "LIMIT " . intval( $offset ) . ", " . intval( $limit );
		$res = $db->query( $query, __METHOD__ );
		while( $row = $db->fetchObject( $res ) ) {
			$result[] = $row;
		}
		return $result;
	}

	function formatResult( $skin, $result ) {
		global $wgLang, $wgContLang;
		$link = "";
		if ( $result !== NULL ) {
			$uid = intval( $result->uid );
			$userName = $result->username;
			$userTitle = Title::makeTitleSafe( NS_USER, $userName );
			$user_link = self::$skin->link( $userTitle, $userName );
			$voice_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_voice_link' ) . ( $this->inverse ? "?" : "" ), array(), array( "id"=>intval( $this->pid), "uid"=>$uid, "action"=>"uvote" ) );
			$link = $user_link . ': ' . $voice_link;
		}
		return $link;
	}

	function linkParameters() {
		$params[ "action" ] = $this->cmd;
		if ( $this->pid !== NULL ) {
			$params[ "id" ] = $this->pid;
		}
		return $params;
	}

}

/* list of users who voted for particular choice of particular proposal of particular question */
class qp_UserCellList extends qp_QueryPage {
	var $cmd;
	var $pid = NULL;
	var $ns, $title, $poll_id, $fragment_id;
	var $question_id, $proposal_id, $cat_id;
	var $inverse = false;

	public function __construct( $cmd, $pid, $question_id, $proposal_id, $cid ) {
		parent::__construct();
		$this->cmd = $cmd;
		$this->question_id = $question_id;
		$this->proposal_id = $proposal_id;
		$this->cat_id = $cid;
		$db = & wfGetDB( DB_SLAVE );
		$qp_poll_desc = $db->tableName( 'qp_poll_desc' );
		$page = $db->tableName( 'page' );
		$query = "SELECT pid, page_namespace as ns, page_title as title, poll_id, fragment_id ";
		$query .= "FROM ($qp_poll_desc, $page) ";
		$query .= "WHERE page_id=article_id AND pid=" . $db->addQuotes( $pid ) . "";
		$res = $db->query( $query, __METHOD__ );
		if ( $row = $db->fetchObject( $res ) ) {
			$this->pid = intval( $row->pid );
			$this->ns = intval( $row->ns );
			$this->title = $row->title;
			$this->poll_id = $row->poll_id;
			$this->fragment_id = $row->fragment_id;
		}
	}

	function getPageHeader() {
		global $wgLang, $wgContLang;
		$link = "";
		$db = & wfGetDB( DB_SLAVE );
		if ( $this->pid !== NULL ) {
			$pollStore = new qp_PollStore( array( 'from'=>'pid', 'pid'=> $this->pid ) );
			if ( $pollStore->pid !== NULL ) {
				$pollStore->loadQuestions();
				$poll_title = Title::makeTitle( intval( $this->ns ), $this->title, qp_Poll::getPollTitleFragment( intval( $this->fragment_id ), '' ) );
				$pagename = htmlspecialchars( $wgContLang->convert( $poll_title->getPrefixedText() ) );
				$pollname = htmlspecialchars( $this->poll_id );
				$goto_link = self::$skin->link( $poll_title, wfMsg( 'qp_source_link' ) );
				$spec = wfMsg( 'qp_spec_line', '"' . $pagename . '"', '"' . $pollname . '"', '', '', '', '' );
				$head[] = PollResults::getPollsLink();
				$head[] = PollResults::getUsersLink();
				$head[] = array( '__tag'=>'span', 'class'=>'head', 0=>wfMsg( 'qp_users_link' ) . ' [ '. $spec . ']' );
				$head[] = ' (' . $goto_link . ')<br />';
				$ques_found = false;
				foreach ( $pollStore->Questions as &$ques ) {
					if ( $ques->question_id == $this->question_id ) {
						$ques_found = true;
						break;
					}
				}
				if ( $ques_found ) {
					$qpa = $this->question_id . '. ' . $ques->CommonQuestion . '<br />';
					if ( array_key_exists( $this->cat_id, $ques->Categories ) ) {
						$categ = &$ques->Categories[ $this->cat_id ];
						$proptext = $ques->ProposalText[ $this->proposal_id ];
						$qpa .= $proptext;
						$qpa .= ( substr( $proptext, strlen( $proptext ) - 1 ) === '?' ) ? ' ' : '? ';
						if ( array_key_exists( 'spanId', $categ ) ) {
							$qpa .= $ques->CategorySpans[ $categ["spanId"] ]["name"] . ' |';
						}
						$qpa .= ' ' . $categ["name"] . '<br />';
						$head[] = array( '__tag'=>'div', 'class'=>'head', 'style'=>'padding-left:2em;', 0=>$qpa );
						$link = qp_Renderer::renderHTMLobject( $head );
					}
				}
			}
		}
		return $link;
	}

	function getIntervalResults( $offset, $limit ) {
		$result = Array();
		$db = & wfGetDB( DB_SLAVE );
		$qp_users = $db->tableName( 'qp_users' );
		$qp_question_answers = $db->tableName( 'qp_question_answers' );
		$query = "SELECT uid, name as username ";
		$query .= "FROM $qp_users ";
		$query .= "WHERE uid IN ";
		$query .= "(SELECT uid FROM $qp_question_answers ";
		$query .= "WHERE pid=" . $db->addQuotes( $this->pid ) . " AND question_id=" . $db->addQuotes( $this->question_id ) . " AND proposal_id=" . $db->addQuotes( $this->proposal_id ) . " AND cat_id=" . $db->addQuotes( $this->cat_id ) . ") ";
		$query .= "LIMIT " . intval( $offset ) . ", " . intval( $limit );
		$res = $db->query( $query, __METHOD__ );
		while( $row = $db->fetchObject( $res ) ) {
			$result[] = $row;
		}
		return $result;
	}

	function formatResult( $skin, $result ) {
		global $wgLang, $wgContLang;
		$link = "";
		if ( $result !== NULL ) {
			$uid = intval( $result->uid );
			$userName = $result->username;
			$userTitle = Title::makeTitleSafe( NS_USER, $userName );
			$user_link = self::$skin->link( $userTitle, $userName );
			$voice_link = self::$skin->link( $this->getTitle(), wfMsg( 'qp_voice_link' ) . ( $this->inverse ? "?" : "" ), array(), array( "id"=>intval( $this->pid), "uid"=>$uid, "action"=>"uvote" ) );
			$link = $user_link . ': ' . $voice_link;
		}
		return $link;
	}

	function linkParameters() {
		$params[ "action" ] = $this->cmd;
		if ( $this->pid !== NULL ) {
			$params[ "id" ] = $this->pid;
			$params[ "qid" ] = $this->question_id;
			$params[ "pid" ] = $this->proposal_id;
			$params[ "cid" ] = $this->cat_id;
		}
		return $params;
	}

}

class qp_Excel {

	static function prepareExcelString( $s ) {
		if ( preg_match( '`^=.?`', $s ) ) {
			return "'" . $s;
		}
		return $s;
	}

	static function writeFormattedTable( $worksheet, $rownum, $colnum, &$table, $format = null ) {
		foreach ( $table as $rnum => &$row ) {
			foreach ( $row as $cnum => &$cell ) {
				if ( is_array( $cell ) ) {
					if ( array_key_exists( "format", $cell ) ) {
						$worksheet->write( $rownum + $rnum, $colnum + $cnum, $cell[ 0 ], $cell[ "format" ] );
					} else {
						$worksheet->write( $rownum + $rnum, $colnum + $cnum, $cell[ 0 ], $format );
					}
				} else {
					$worksheet->write( $rownum + $rnum, $colnum + $cnum, $cell, $format );
				}
			}
		}
	}

}