<?php
/**
 * ***** BEGIN LICENSE BLOCK *****
 * This file is part of QPoll.
 * Uses parts of code from Quiz extension (c) 2007 Louis-Rémi BABE. All rights reserved.
 *
 * QPoll is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * QPoll is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with QPoll; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ***** END LICENSE BLOCK *****
 *
 * QPoll is a poll tool for MediaWiki.
 * 
 * To activate this extension :
 * * Create a new directory named QPoll into the directory "extensions" of MediaWiki.
 * * Place the files from the extension archive there.
 * * Add this line at the end of your LocalSettings.php file :
 * require_once "$IP/extensions/QPoll/qp_user.php";
 * 
 * @version 0.3.0
 * @link http://www.mediawiki.org/wiki/Extension:QPoll
 * @author QuestPC <questpc@rambler.ru>
 */

if ( !defined( 'MEDIAWIKI' ) ) {
	die( "This file is part of the QPoll extension. It is not a valid entry point.\n" );
}

qp_CheckRequiredClassesAndMethods();

define( 'QP_CSS_ERROR_COLOR1' , "LightYellow" );
define( 'QP_CSS_ERROR_COLOR2', "#D700D7" );
define( 'QP_CSS_ERROR_STYLE', 'background-color: ' . QP_CSS_ERROR_COLOR1 . ';' );

/**
 * Extension's parameters.
 */
$wgExtensionCredits['parserhook'][] = array(
	'name'						=> 'QPoll',
	'version'					=> '0.3.0',
	'author'					=> 'QuestPC',
	'url'							=> 'http://www.mediawiki.org/wiki/Extension:QPoll',
	'description'			=> 'Allows creation of polls',
	'descriptionmsg'	=> 'qp_desc'
);
$wgExtensionCredits['specialpage'][] = array(
	'name' => 'QPoll results page',
	'version' => '0.3.0',
	'author' => 'QuestPC',
	'url' => 'http://www.mediawiki.org/wiki/Extension:QPoll',
	'description' => 'QPoll extension [[Special:PollResults]] page for viewing results of the polls',
	'descriptionmsg' => 'qp_desc',
);

/**
 * Add this extension to the mediawiki's extensions list.
 */
$qp_ExtDir = str_replace( "\\", "/", dirname(__FILE__) ); // filesys path with windows path fix
$qp_top_dir = array_pop( explode( '/', $qp_ExtDir ) );
$qp_ScriptPath = $wgScriptPath . '/extensions' . ( ( $qp_top_dir == 'extensions' ) ? '' : '/' . $qp_top_dir ); // apache virtual path

$wgExtensionMessagesFiles['QPoll'] = $qp_ExtDir . '/qp_i18n.php';
$wgAutoloadClasses['PollResults'] = $qp_ExtDir . '/qp_results.php';
$wgAutoloadClasses['qp_QueryPage'] = $qp_ExtDir . '/qp_results.php';
$wgSpecialPages['PollResults'] = array('PollResults');

if ( defined( 'MW_SUPPORTS_PARSERFIRSTCALLINIT' ) ) {
	$wgHooks['ParserFirstCallInit'][] = 'wfQPollExtension';
} else {
	die( "QPoll extension requires ParserFirstCallInit hook.\nPlease upgrade your MediaWiki installation first.\n" );
	$wgExtensionFunctions[] = 'wfQPollExtension';
}

$wgHooks['ParserFirstCallInit'][] = 'qp_Poll::resetPollFragmentID';
$wgHooks['LanguageGetMagic'][]       = 'qp_FunctionsHook::languageGetMagic';
$wgHooks['LoadAllMessages'][] = 'qp_Poll::loadMessages';

function qp_CheckRequiredClassesAndMethods() {
	$required_classes_and_methods = array(
		array( "Linker"=>"link" ),
		array( "OutputPage"=>"isPrintable" ),
		array( "Parser"=>"fetchTemplate" ),
		array( "Parser"=>"getTitle" ),
		array( "Parser"=>"setHook" ),
		array( "StringUtils"=>"escapeRegexReplacement" ),
		array( "Title"=>"getArticleID" ),
		array( "Title"=>"getPrefixedText" ),
		array( "Title"=>"makeTitle" ),
		array( "Title"=>"makeTitleSafe" ),
		array( "Title"=>"newFromID" )
	);
	foreach ( $required_classes_and_methods as &$check ) {
		list( $object, $method ) = each( $check );
		if ( !method_exists( $object, $method ) ) {
			die( "QPoll extension requires " . $object . "::" . $method . " to be available.<br />\n" .
				"Your version of MediaWiki is incompatible with this extension.\n" );
		}
	}
}

/**
 * Register the extension with the WikiText parser.
 * The tag is: <qpoll>
 */
function wfQPollExtension() {
	global $wgParser;
	global $wgExtensionCredits;
	global $wgQPollFunctionHook;
	$wgParser->setHook("qpoll", "qp_RenderPoll");
	$wgQPollFunctionsHook = new qp_FunctionsHook();
	$wgParser->setFunctionHook( 'qpuserchoice', array( &$wgQPollFunctionsHook, 'qpuserchoice' ), SFH_OBJECT_ARGS );
	return true;
}

/**
 * Call the poll parser on an input text.
 * 
 * @param  $input				Text between <qpoll> and </qpoll> tags, in QPoll syntax.
 * @param  $argv				An array containing any arguments passed to the extension
 * @param  &$parser				The wikitext parser.
 * 
 * @return 						An HTML poll.
 */

/* @param  $input				Text between <qpoll> and </qpoll> tags, in QPoll syntax. */
function qp_RenderPoll($input, $argv, &$parser) {
	$parser->disableCache();
	$qpoll = new qp_Poll($argv, $parser);
	return $qpoll->parsePoll($input);
}

/**
 * Processes poll markup
 */
class qp_Poll {
	/**#@+
	 * @public
	 */

	static $skin;
	static $sFragmentId = 0;
	/**#@- */

	static $sPrevPollIDs = Array(); // used to check uniqueness of PollId on the page

	var $username;
	// params of tag
	var $wikiTag = "qpoll";
	var $showResults = false;
	var $dependanceStr = '';

	// qp_pollStore object will be used to transfer poll data from/to DB
	var $pollStore;

	/** 
	 * Constructor
	 * 
	 * @public
	 */
	function __construct($argv, &$parser) {
		global $wgUser, $wgRequest, $wgLanguageCode;
		global $qp_enable_showresults;
		$this->mParser = $parser;
		$this->mRequest = &$wgRequest;
		# Determine which messages will be used, according to the language.
		self::loadMessages();
		# load current skin
		if ( self::$skin === NULL ) {
			self::$skin = $wgUser->getSkin();
		}
		# Allot a unique fragment id (# part of URI) to address the poll on the page.
		$this->mFragmentId = self::$sFragmentId;
		self::$sFragmentId++;
		# Reset the unique identifier of the questions.
		$this->mQuestionId = 0;
		# Initialize various parameters
		$this->mState = "";
		$this->showResults = array_key_exists('showresults', $argv) && ($qp_enable_showresults === true);
		$this->username = qp_Poll::currentUserName();
		if ( array_key_exists('dependance', $argv) ) {
			$this->dependanceStr = trim( $argv[ 'dependance' ] );
		}
		# every poll on the page should have unique poll id, to minimize the risk of collisions
		# it is required to be set manually via id="num" parameter
		$this->mPollId = array_key_exists('id', $argv) ? $argv['id'] : NULL;
		# Determine if this poll is being corrected or not, according to the pollId
		$this->mBeingCorrected = ( $wgRequest->getVal('pollId') == $this->mPollId ) ? true : false;
		$this->perRow = intval( array_key_exists( 'perrow', $argv ) ? $argv['perrow'] : 1 );
		if ( $this->perRow < 1 )
			$this->perRow = 1;
		$this->currCol = $this->perRow;
	}

	# title fragment part (GET/POST)
	static function getPollTitleFragment( $fragment_id, $dash = '#' ) {
		return $dash . 'qp' . intval( $fragment_id );
	}

	static function fatalError( $msg ) {
		return '<div class="qpoll"><div class="fatalerror">' . $msg . '</div></div>';
	}
	
	static function resetPollFragmentId() {
		self::$sFragmentId = 0;
		return true;
	}
	
	static function loadMessages() {
			static $messagesLoaded = false;
			if ( $messagesLoaded ) return true;
			$messagesLoaded = true;
		wfLoadExtensionMessages('QPoll');
		return true;
	}	

	static function currentUserName() {
		global $qp_AnonForwardedFor;
		global $wgUser, $wgSquidServers;
		global $wgUsePrivateIPs;
		if ( $qp_AnonForwardedFor === true && $wgUser->isAnon() ) {
			/* collect the originating IPs 
				borrowed from ProxyTools::wfGetIP
				bypass trusted proxies list check */
			# Client connecting to this webserver
			if ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
				$ipchain = array( IP::canonicalize( $_SERVER['REMOTE_ADDR'] ) );
			} else {
				# Running on CLI?
				$ipchain = array( '127.0.0.1' );
			}
			$ip = $ipchain[0];

			# Append XFF on to $ipchain
			$forwardedFor = wfGetForwardedFor();
			if ( isset( $forwardedFor ) ) {
				$xff = array_map( 'trim', explode( ',', $forwardedFor ) );
				$xff = array_reverse( $xff );
				$ipchain = array_merge( $ipchain, $xff );
			}
			$username = "";
			foreach ( $ipchain as $i => $curIP ) {
				if( $wgUsePrivateIPs || IP::isPublic( $curIP ) ) {
					$username .= IP::canonicalize( $curIP ) . '/';
				}
			}
			if ( username != "" ) {
				# remove trailing slash
				$username = substr( $username, 0, strlen( $username ) - 1 );
			} else {
				$username .= IP::canonicalize( $ipchain[0] );
			}
		} else {
			$username = $wgUser->getName();
		}
		return $username;
	}

	/**
	 * Convert the input text to an HTML output.
	 * 
	 * @param  $input				Text between <qpoll> and </qpoll> tags, in QPoll syntax.
	 */
	function parsePoll($input) {
		global $qp_ExtDir, $qp_ScriptPath;
		global $wgOut, $wgRequest;
		# Ouput the style and the script to the header once for all.
		if( $this->mFragmentId == 0 ) {
			global $wgContLang;
			global $wgJsMimeType, $wgOut;
			$head  = '<style type="text/css">' . "\n";
			$head .= '.qpoll .fatalerror { border: 1px solid gray; padding: 4px; ' . QP_CSS_ERROR_STYLE . ' }' . "\n";
			$head .= '</style>' . "\n";
			$head .= '<script type="' . $wgJsMimeType . '" src="' . $qp_ScriptPath . '/qp_user.js"></script>' . "\n";
			$wgOut->addScript( $head );
			$wgOut->addExtensionStyle( $qp_ScriptPath . '/qp_user.css' );
			if ( $wgContLang->isRTL() ) {
				$wgOut->addExtensionStyle( $qp_ScriptPath . '/qp_user_rtl.css' );
			}
		}
		# check the headers for errors
		if ( $this->mPollId == NULL ) {
			$this->mState == "error";
			return qp_Poll::fatalError( wfMsgHtml( 'qp_error_no_poll_id' ) );
		}
		if ( !qp_Poll::isUniquePollId( $this->mPollId ) ) {
			$this->mState == "error";
			return qp_Poll::fatalError( wfMsgHtml( 'qp_error_already_used_poll_id', $this->mPollId ) );
		}
		if ( $this->dependanceStr != '' ) {
			$depSplit = qp_Poll::splitDependanceString( $this->dependanceStr );
			if ( is_array( $depSplit ) ) {
				$this->dependanceStr = qp_Poll::buildDependanceString( $depSplit );
			} else {
				return qp_Poll::fatalError( wfMsgHtml( 'qp_error_invalid_dependance_value', $this->mPollId, $dependanceStr ) );
			}
		}
		if ( ( $dependanceResult = $this->checkDependance( $this->dependanceStr ) ) !== true ) {
			# return an error string
			# here we create a pollstore only to update fragment_id and dependance, in case these were changed
			$this->pollStore = new qp_PollStore( array(
				'from'=>'poll_get',
				'poll_id'=>$this->mPollId,
				'fragment_id'=>$this->mFragmentId,
				'dependance'=>$this->dependanceStr ) );
			return $dependanceResult;
		}
		self::addPollId( $this->mPollId ); // add current poll id to the static list of poll ids on this page
		if ( $this->mBeingCorrected ) {
			$this->pollStore = new qp_PollStore( array(
			'from'=>'poll_post',
			'poll_id'=>$this->mPollId,
			'fragment_id'=>$this->mFragmentId,
			'dependance'=>$this->dependanceStr ) );
			$this->pollStore->loadQuestions();
		} else {
			$this->pollStore = new qp_PollStore( array(
				'from'=>'poll_get',
				'poll_id'=>$this->mPollId,
				'fragment_id'=>$this->mFragmentId,
				'dependance'=>$this->dependanceStr ) );
			$this->pollStore->loadQuestions();
			$this->pollStore->setLastUser( $this->username );
			# to show previous choice of current user, if that's available
			$this->pollStore->loadUserVote();
		}
		if ( $this->showResults ) {
			$this->pollStore->loadAllVotes();
			$this->pollStore->calculateStatistics();
		}
		# Process the input
		$input = $this->parseQuestions( $input );
		# check, whether the poll is fine?
		$this->pollStore->stateComplete();
		# user votes (when poll is fine)
		$this->pollStore->setLastUser( $this->username );
		$this->pollStore->setUserVote();
		$user_agent = wfGetAgent();
		# redirect is temporary disabled in Opera, cannot get to work
		if ( $this->pollStore->voteDone && stristr( $user_agent, "Opera" ) === false ) {
			# redirect in case of voting, otherwise stats display will not be updated (showresults parameter)
			$wgRequest->response()->header( 'HTTP/1.0 302 Moved Temporarily' );
			$wgRequest->response()->header( 'Location: ' . wfExpandUrl( self::getPollTitleFragment( $this->mFragmentId ) ) );
			return;
		}
		# Generates the output.
		$qpoll_div = array( '__tag'=>'div', 'class'=>'qpoll' );
		$qpoll_div[] = array( '__tag'=>'a', 'name'=>self::getPollTitleFragment( $this->mFragmentId, '' ), 0=>'' );
		$qpoll_form = array( '__tag'=>'form', 'id'=>'poll' . $this->mFragmentId, 'method'=>'post', 'action'=>self::getPollTitleFragment( $this->mFragmentId ), '__end'=>"\n" );
		$qpoll_div[] = &$qpoll_form;
		# Determine the content of the settings table.
		$settings = Array();
		if ( $this->mState != '' ) {
			$settings[0][] = array( '__tag'=>'td', 'class'=>'margin', 'style'=>'background: ' . QP_CSS_ERROR_COLOR2 . ';' );
			$settings[0][] = array( '__tag'=>'td', 0=>wfMsgHtml( 'qp_result_' . $this->mState ) );
		}
		# Build the settings table.
		if ( count( $settings ) > 0 ) {
			$settingsTable = array( '__tag'=>'table', 'class'=>'settings', '__end'=>"\n" );
			foreach($settings as $settingsTr) {
				$settingsTable[] = array( '__tag'=>'tr', 0=> $settingsTr, '__end'=>"\n" );
			}		
			$qpoll_form[] = &$settingsTable;
		}
		$qpoll_form[] = array( '__tag'=>'input', 'type'=>'hidden', 'name'=>'pollId', 'value'=>$this->mPollId );
		$qpoll_form[] = array( '__tag'=>'div', 'class'=>'pollQuestions', 0=>$input );
		$submitBtn = array( '__tag'=>'input', 'type'=>'submit' );
		$submitMsg = 'qp_vote_button';
		if ( $this->pollStore->isAlreadyVoted() ) {
			$submitMsg = 'qp_vote_again_button';
		}
		if ( $this->mBeingCorrected ) {
			if ( $this->pollStore->getState() == "complete" ) {
				$submitMsg = 'qp_vote_again_button';
			}
		} else {
			if ( $this->pollStore->getState() == "error" ) {
				$submitBtn[ 'disabled' ] = 'disabled';
			}
		}
		// disable submit button in preview mode & printable version
		if ( $wgRequest->getVal( 'action' ) == 'submit' ||
				$wgOut->isPrintable() ) {
			$submitBtn[ 'disabled' ] = 'disabled';
		}
		$submitBtn[ 'value' ] = wfMsgHtml( $submitMsg );
		$p = array( '__tag'=>'p' );
		$p[] = $submitBtn;
		$qpoll_form[] = &$p;
		return qp_Renderer::renderHTMLobject( $qpoll_div );
	}

	// $dependanceStr should not be empty!
	static function splitDependanceString( $dependanceStr ) {
		global $wgArticle;
		preg_match( '`^(.*?)#(.*?)$`u', $dependanceStr, $matches );
		if ( count( $matches ) == 3 ) {
			$depTitleStr = trim( str_replace( "''", '"', $matches[ 1 ] ) );
			$depPollId = trim( $matches[ 2 ] );
			if ( $depPollId == '' ) {
				return false;
			}
			if ( $depTitleStr == '' ) {
				# dependancy poll is located at the current page
				$depTitleStr = $wgArticle->getTitle()->getPrefixedText();
			}
		} else {
			return false;
		}
		return array( $depTitleStr, $depPollId );
	}

	static function buildDependanceString( $depSplit ) {
		if ( is_array( $depSplit ) ) {
			return $depSplit[ 0 ] . '#' . $depSplit[ 1 ];
		} else {
			return $depSplit;
		}
	}

	# check whether the poll is dependant on other polls
	# returns true when dependance is fulfilled, error message otherwise
	private function checkDependance( $dependanceStr, $nonVotedDepLink = false) {
		# check the headers for dependance to other polls
		if ( $dependanceStr !== '' ) {
			$depSplit = qp_Poll::splitDependanceString( $dependanceStr );
			if ( is_array( $depSplit ) ) {
				list( $depTitleStr, $depPollId ) = $depSplit;
				$depTitle = Title::newFromURL( $depTitleStr );
				$depTitleStr = $depTitle->getPrefixedText();
				$depLink = self::$skin->link( $depTitle, $depTitleStr );
				if ( $depTitle !== NULL ) {
					$depArticleId = intval( $depTitle->getArticleID() );
					if ( $depArticleId > 0 ) {
						$depPollStore = new qp_PollStore( array(
							'from'=>'poll_get',
							'title'=>$depTitle,
							'poll_id'=>$depPollId ) );
						if ( $depPollStore->pid === NULL ) {
							return qp_Poll::fatalError( wfMsgHtml( 'qp_error_missed_dependance_poll', $this->mPollId, $depLink, $depPollId ) );
						}
						$depPollStore->loadQuestions();
						$depPollStore->setLastUser( $this->username );
						$depPollStore->loadUserVote();
						$depTitle = $depPollStore->getTitle();
						$depLink = self::$skin->link( $depTitle, $depTitle->getPrefixedText() . ' (' . $depPollStore->mPollId . ')' );
						if ( $depPollStore->isAlreadyVoted() ) {
							if ( $depPollStore->dependanceStr === '' ) {
								if ( $nonVotedDepLink === false ) {
									return true;
								} else {
									return qp_Poll::fatalError( wfMsgHtml( 'qp_error_vote_dependance_poll', $nonVotedDepLink ) );
								}
							} else {
								return $this->checkDependance( $depPollStore->dependanceStr, $nonVotedDepLink );
							}
						} else {
							if ( $depPollStore->dependanceStr === '' ) {
								return qp_Poll::fatalError( wfMsgHtml( 'qp_error_vote_dependance_poll', $depLink ) );
							} else {
								return $this->checkDependance( $depPollStore->dependanceStr, $depLink );
							}
						}
					} else {
						return qp_Poll::fatalError( wfMsgHtml( 'qp_error_missed_dependance_title', $this->mPollId, $depLink, $depPollId ) );
					}
				} else {
					return qp_Poll::fatalError( wfMsgHtml( 'qp_error_missed_dependance_title', $this->mPollId, $depLink, $depPollId ) );
				}
			} else {
				return qp_Poll::fatalError( wfMsgHtml( 'qp_error_invalid_dependance_value', $this->mPollId, $dependanceStr ) );
			}
		} else {
			return true;
		}
	}

	/**
	 * Replace questions from QPoll syntax to HTML.
	 * 
	 * @param  $input				A question in QPoll syntax.
	 */
	function parseQuestions($input) {
		$write_row = Array();
		$write_col = Array();
		$splitPattern = '`(^|\n[ \t]*)\n\{`u';
		$unparsedQuestions = preg_split($splitPattern, $input, -1, PREG_SPLIT_NO_EMPTY );
		$questionPattern = '`(.*?[^|\}])\}[ \t]*(\n(.*)|$)`su';
		foreach($unparsedQuestions as $unparsedQuestion) {
			# If this "unparsedQuestion" is not a full question,
			# we put the text into a buffer to add it at the beginning of the next question.
			if( !empty( $buffer ) )
				$unparsedQuestion = "$buffer\n\n{".$unparsedQuestion;
			if( preg_match( $questionPattern, $unparsedQuestion, $matches ) ) {
				$buffer = "";
				if ( $this->perRow > 1 ) {
					$write_col[] = array( '__tag'=>'td', 'valign'=>'top', 0=>$this->parseQuestion($matches), '__end'=>"\n" );
					if ( $this->currCol == 1 ) {
						$write_row[] = array( '__tag'=>'tr', 0=>$write_col, '__end'=>"\n" );
						$write_col = Array();
					}
					if ( --$this->currCol < 1 ) {
						$this->currCol = $this->perRow;
					}
				} else {
					$write_row[] = $this->parseQuestion($matches);
				}
			} else {
				$buffer = $unparsedQuestion;
			}
		}
		if ( $this->perRow > 1 && $this->currCol != $this->perRow ) {
			# add last incomplete row
			$write_row[] = array( '__tag'=>'tr', '__end'=>"\n", 0=>$write_col );
		}
		if ( $this->perRow > 1 ) {
			$question_table = array( '__tag'=>'table', 0=>array( '__tag'=>'tbody', 0=>&$write_row, '__end'=>"\n" ), '__end'=>"\n" );
			return qp_Renderer::renderHTMLobject( $question_table );
		} else {
			return qp_Renderer::renderHTMLobject( $write_row );
		}
	}
	
	/**
	 * Convert a question on the page from QPoll syntax to HTML
	 * 
	 * @param  $matches				Elements matching $questionPattern.
	 * 								$matches[1] is the question header.
	 * 								$matches[3] is the question object.
	 */
	function parseQuestion($matches) {
		$question = new qp_Question( $this->mBeingCorrected, $this->mQuestionId, $this->mParser, $this->showResults );
		$tags = array( '__tag'=>'div', '__end'=>"\n", 'class'=>'question',
			0=>array( '__tag'=>'div', '__end'=>"\n", 'class'=>'header',
				0=>array( '__tag'=>'span', 'class'=>'questionId', 0=>++$this->mQuestionId )
			),
			1=>$question->parseHeader( $matches[ 1 ] ) );
		# load previous user choice, when it's available and DB header is compatible with parsed header
		if ( !isset( $matches[ 3 ] ) ||
				!method_exists( $question, $question->mType . 'ParseObject' ) ||
				!method_exists( $question, $question->mType . 'ParseObjectHeader' )) {
			$question->setState( "error" );
			$buffer = wfMsgHtml( 'qp_error_invalid_question_header' );
		} else {
			# parse the object's header and set object body "raws"
			call_user_func( array( $question, $question->mType . 'ParseObjectHeader' ), $matches[ 3 ] );
			$this->pollStore->getQuestionAnswer( $this->mQuestionId, $question );
			# parse the object's body store the parsed object into a buffer to determine some parameters before outputing it.
			$buffer = call_user_func( array( $question, $question->mType . 'ParseObject' ) );
		}
		if ( $this->mBeingCorrected ) {
			if ( $question->getState() == "" ) {
				$this->pollStore->postQuestion( $this->mQuestionId, $question );
			} else {
				# http post: not every proposals were answered: do not update DB
				$this->pollStore->stateIncomplete();
			}
		} else {
			# this is the get, not the post: do not update DB
			if ( $question->getState() == "" ) {
				$this->pollStore->stateIncomplete();
			} else {
				# http get: invalid question syntax, parse errors will cause submit button disabled
				$this->pollStore->stateError();
			}
		}
		$output_table = array( '__tag'=>'table', '__end'=>"\n", 'class'=>'object' );
		# Determine the side border color the question.
		if ( $question->getState() != "" ) {
			global $wgContLang;
			$style = $wgContLang->isRTL() ? 'border-right' : 'border-left';
			$style .= ': 3px solid ' . QP_CSS_ERROR_COLOR2 . ';';
			$output_table[ 'style' ] = $style;
			$this->mState = $question->getState();
		}
		$output_table[] = array( '__tag'=>'tbody', '__end'=>"\n", 0=>&$buffer );
		$tags[] = &$output_table;
		return qp_Renderer::renderHTMLobject( $tags );
	}

	function addPollId ( $pollId ) {
		self::$sPrevPollIDs[] = $pollId;
	}

	function isUniquePollId( $pollId ) {
		return !in_array( $pollId, self::$sPrevPollIDs );
	}

}

/* poll's single question data object RAM storage
 */
class qp_QuestionData {

	// DB index (with current scheme is non-unique)
	var $question_id = NULL;
	// common properties
	var $type;
	var $CommonQuestion;
	var $Categories;
	var $CategorySpans;
	var $ProposalText;
	var $ProposalCategoryId;
	var $alreadyVoted = false; // whether the selected user has already voted this question ?
	// statistics storage
	var $Votes = NULL;
	var $Percents = NULL;

	function __construct( $argv ) {
		if ( array_key_exists( "from", $argv ) ) {
			switch ( $argv[ "from" ] ) {
				case 'postdata' :
						$this->type = $argv[ "type" ];
						$this->CommonQuestion = $argv[ "common_question" ];
						$this->Categories = $argv[ "categories" ];
						$this->CategorySpans = $argv[ "category_spans" ];
						$this->ProposalText = $argv[ "proposal_text" ];
						$this->ProposalCategoryId = $argv[ "proposal_category_id" ];
					break;
				case 'qid' :
						$this->question_id=$argv[ "qid" ];
						$this->type = $argv[ "type" ];
						$this->CommonQuestion = $argv[ "common_question" ];
						$this->Categories = Array();
						$this->CategorySpans = Array();
						$this->ProposalText = Array();
						$this->ProposalCategoryId = Array();
					break;
			}
		}
	}

	// integrate spans into categories
	function removeSpans() {
		if ( count( $this->CategorySpans ) > 0 ) {
			foreach( $this->Categories as &$Cat ) {
				if ( array_key_exists( "spanId", $Cat ) ) {
					$Cat["name"] = $this->CategorySpans[ $Cat["spanId"] ]["name"] . " | " . $Cat["name"];
					unset( $Cat["spanId"] );
				}
			}
			unset( $this->CategorySpans );
			$this->CategorySpans = Array();
		}
	}

	// restore spans from categories
	function restoreSpans() {
		if ( count( $this->CategorySpans ) == 0 ) {
			$prevSpanName = "";
			$spanId = -1;
			foreach( $this->Categories as &$Cat ) {
				$a = explode( " | ", $Cat["name"] );
				if ( count( $a ) > 1 ) {
					if ( $prevSpanName != $a[0] ) {
						$spanId++;
						$prevSpanName = $a[0];
						$this->CategorySpans[ $spanId ]["count"] = 0;
					}
					$Cat["name"] = $a[1];
					$Cat["spanId"] = $spanId;
					$this->CategorySpans[ $spanId ]["name"] = $a[0];
					$this->CategorySpans[ $spanId ]["count"]++;
				} else {
					$prevSpanName = "";
				}
			}
		}
	}

	// check whether the previousely stored poll header is compatible with the one defined in the page
	// used to reject previous vote in case the header is incompatble
	function isCompatible( &$question ) {
		if ( $question->mType != $this->type ) {
			return false;
		}
		if ( count( $question->mCategorySpans ) != count( $this->CategorySpans ) ) {
			return false;
		}
		foreach ( $question->mCategorySpans as $spanidx => &$span ) {
			if ( !isset( $this->CategorySpans[ $spanidx ] ) ||
					$span[ "count" ] != $this->CategorySpans[ $spanidx ][ "count" ] ) {
				return false;
			}
		}
//	$question->mPrevCategories = $this->Categories;
//	$question->mPrevCategorySpans = $this->CategorySpans;
		return true;
	}

	function getAnswer( &$question ) {
		$question->alreadyVoted = $this->alreadyVoted;
		$question->mPrevProposalCategoryId = $this->ProposalCategoryId;
		if ( isset( $this->Percents ) && is_array( $this->Percents ) ) {
			$question->Percents = $this->Percents;
		}
	}

}

/* poll storage and retrieval using DB
 * one poll may contain several questions
 */
class qp_PollStore {

	static $db = NULL;
	// DB keys
	var $pid = NULL;
	var $last_uid = NULL;
	// common properties
	var $mArticleId = NULL;
	var $mPollId = NULL;
	var $mFragmentId = NULL; // anchor #link on the page
	var $dependanceStr = NULL; // dependance from other poll string in the following format: "page#otherpollid"
	var $Questions = NULL; // array of QuestionData instances (data from/to DB)
	var $mCompletedPostData;
	var $voteDone = false;

 /* $argv[ 'from' ] indicates type of construction, other elements of $argv vary according to 'from'
  */
	function __construct( $argv = NULL ) {
		global $wgParser;
		if ( self::$db == NULL )
			self::$db = & wfGetDB( DB_SLAVE );
		if ( is_array($argv) && array_key_exists( "from", $argv ) ) {
			$this->Questions = Array();
			$this->mCompletedPostData = "NA";
			$this->pid = NULL;
			$is_post = false;
			switch ( $argv[ 'from' ] ) {
				case 'poll_post' :
					$is_post= true;
				case 'poll_get' :
					if ( array_key_exists( 'title', $argv ) ) {
						$title = $argv[ 'title' ];
					} else {
						$title = $wgParser->getTitle();
					}
					$this->mArticleId = $title->getArticleID();
					$this->mPollId = $argv[ 'poll_id' ];
					if ( array_key_exists( 'fragment_id', $argv ) ) {
						$this->mFragmentId = $argv[ 'fragment_id' ];
					}
					if ( array_key_exists( 'dependance', $argv ) &&
							$argv[ 'dependance' ] !== false ) {
						$this->dependanceStr = $argv[ 'dependance' ];
					}
					if ( $is_post ) {
						$this->setPid();
					} else {
						$this->loadPid();
					}
					break;
				case 'pid' :
					if ( array_key_exists( 'pid', $argv ) ) {
						$pid = intval( $argv[ 'pid' ] );
						$res = self::$db->select( 'qp_poll_desc',
							array( 'article_id', 'poll_id','fragment_id', 'dependance' ),
							array( 'pid'=>$pid ),
							__METHOD__ . ":create from pid" );
						$row = self::$db->fetchObject( $res );
						if ( $row!=false ) {
							$this->pid = $pid;
							$this->mArticleId = $row->article_id;
							$this->mPollId = $row->poll_id;
							$this->mFragmentId = $row->fragment_id;
							$this->dependanceStr = $row->dependance;
						}
					}
					break;
			}
		}
	}

	function getPollId() {
		return $this->mPollId;
	}

	# returns Title object, to get a URI path, use Title::getFullText()/getPrefixedText() on it
	function getTitle() {
		$res = NULL;
		if ( $this->mArticleId !==NULL && $this->mFragmentId !== NULL ) {
			$res = Title::newFromID( $this->mArticleId );
			$res->setFragment( qp_Poll::getPollTitleFragment( $this->mFragmentId ) );
		}
		return $res;
	}

	// will work only after successful loadUserVote()
	function isAlreadyVoted() {
		if ( is_array( $this->Questions ) && count( $this->Questions > 0 ) ) {
			foreach( $this->Questions as &$qdata ) {
				if ( $qdata->alreadyVoted )
					return true;
			}
		}
		return false;
	}

	function loadQuestions() {
		if ( $this->pid !== NULL ) {
			$res = self::$db->select( 'qp_question_desc',
				array( 'question_id', 'type', 'common_question' ),
				array( 'pid'=>$this->pid ),
				__METHOD__ );
			while ( $row = self::$db->fetchObject( $res ) ) {
				$question_id = intval( $row->question_id );
				$this->Questions[ $question_id ] = new qp_QuestionData( array(
					'from'=>'qid',
					'qid'=>$question_id,
					'type'=>$row->type,
					'common_question'=>$row->common_question ) );
			}
			$this->getCategories();
			$this->getProposalText();
		}
	}

	// single user vote will be written into self::Questions[]->ProposalCategoryId
	// returns true on success, false on failure
	function loadUserVote() {
		if ( $this->pid !== NULL && $this->last_uid !== NULL &&
				is_array( $this->Questions ) && count( $this->Questions > 0 ) ) {
			$res = self::$db->select( 'qp_question_answers',
				array( 'question_id', 'proposal_id', 'cat_id' ),
				array( 'pid'=>$this->pid, 'uid'=>$this->last_uid ),
				__METHOD__ . ':load one user single poll vote' );
			while ( $row = self::$db->fetchObject( $res ) ) {
				$question_id = intval( $row->question_id );
				if ( array_key_exists( $question_id, $this->Questions ) ) {
					$qdata = &$this->Questions[ $question_id ];
					$qdata->alreadyVoted = true;
					$qdata->ProposalCategoryId[ intval( $row->proposal_id ) ][] = intval( $row->cat_id );
				}
			}
		}
		return true;
	}

	function loadAllVotes() {
		if ( $this->pid !== NULL &&
				is_array( $this->Questions ) && count( $this->Questions > 0 ) ) {
			$res = self::$db->select( 'qp_question_answers',
				array( 'count(uid)', 'question_id', 'proposal_id', 'cat_id' ),
				array( 'pid'=>$this->pid ),
				__METHOD__ . ':load single poll count of user votes',
				array( 'GROUP BY' => 'question_id,proposal_id,cat_id' ) );
			while ( $row = self::$db->fetchRow( $res ) ) {
				$question_id = intval( $row[ "question_id" ] );
				$propkey = intval( $row[ "proposal_id" ] );
				$catkey = intval( $row[ "cat_id" ] );
				if ( array_key_exists( $question_id, $this->Questions ) ) {
					$qdata = &$this->Questions[ $question_id ];
					if ( !is_array( $qdata->Votes ) ) {
						$qdata->Votes = Array();
					}
					if ( !array_key_exists( $propkey, $qdata->Votes ) ) {
						$qdata->Votes[ $propkey ] = array_fill( 0, count( $qdata->Categories ), 0 );
					}
					$qdata->Votes[ $propkey ][ $catkey ] = intval( $row[ "count(uid)" ] );
				}
			}
		}
	}

	function totalUsersAnsweredQuestion( &$qdata ) {
		$result = 0;
		if ( $this->pid !== NULL ) {
			$res = self::$db->select( 'qp_question_answers',
				array( 'count(distinct uid)' ),
				array( 'pid'=>$this->pid, 'question_id'=>$qdata->question_id ),
				__METHOD__ );
			if ( $row = self::$db->fetchRow( $res ) ) {
				$result = intval( $row[ "count(distinct uid)" ] );
			}
		}
		return $result;
	}

	function calculateStatistics() {
		foreach( $this->Questions as &$qdata ) {
			$this->calculateQuestionStatistics( $qdata );
		}
	}

	private function calculateQuestionStatistics( &$qdata ) {
		if ( in_array( $qdata->type, array( "singleChoicePoll", "multipleChoicePoll" ) ) ) {
			$qdata->restoreSpans();
			$spansUsed = count( $qdata->CategorySpans ) > 0 ;
			foreach ( $qdata->ProposalText as $propkey => $proposal_text ) {
				if ( isset( $qdata->Votes[ $propkey ] ) ) {
					$votes_row = &$qdata->Votes[ $propkey ];
					if ( $qdata->type == "singleChoicePoll" ) {
						if ( $spansUsed ) {
							$row_totals = array_fill( 0, count( $qdata->CategorySpans ), 0 );
						} else {
							$votes_total = 0;
						}
						foreach( $qdata->Categories as $catkey => $cat ) {
							if ( isset( $votes_row[ $catkey ] ) ) {
								if ( $spansUsed ) {
									$row_totals[ intval( $cat[ "spanId" ] ) ] += $votes_row[ $catkey ];
								} else {
									$votes_total += $votes_row[ $catkey ];
								}
							}
						}
					} else {
						$votes_total = $this->totalUsersAnsweredQuestion( $qdata );
					}
					foreach( $qdata->Categories as $catkey => $cat ) {
						$num_of_votes = '';
						if ( isset( $votes_row[ $catkey ] ) ) {
							$num_of_votes = $votes_row[ $catkey ];
							if ( $spansUsed ) {
								if ( isset( $qdata->Categories[ $catkey ][ "spanId" ] ) ) {
									$votes_total = $row_totals[ intval( $qdata->Categories[ $catkey ][ "spanId" ] ) ];
								}
							}
						}
						$qdata->Percents[ $propkey ][ $catkey ] = ( $votes_total > 0 ) ? (float) $num_of_votes / (float) $votes_total : 0.0;
					}
				}
			}
		}
	}

	private function getCategories() {
		$res = self::$db->select( 'qp_question_categories',
				array( 'question_id', 'cat_id', 'cat_name' ),
				array( 'pid'=>$this->pid ),
				__METHOD__ );
		while ( $row = self::$db->fetchObject( $res ) ) {
			$question_id = intval( $row->question_id );
			$cat_id = intval( $row->cat_id );
			if ( array_key_exists( $question_id, $this->Questions ) ) {
				$qdata = &$this->Questions[ $question_id ];
				$qdata->Categories[ $cat_id ][ "name" ] = $row->cat_name;
			}
		}
		foreach ( $this->Questions as &$qdata ) {
			$qdata->restoreSpans();
		}
	}

	private function getProposalText() {
		$res = self::$db->select( 'qp_question_proposals',
			array( 'question_id', 'proposal_id', 'proposal_text' ),
			array( 'pid'=>$this->pid ),
				__METHOD__ );
		while ( $row = self::$db->fetchObject( $res ) ) {
			$question_id = intval( $row->question_id );
			$proposal_id = intval( $row->proposal_id );
			if ( array_key_exists( $question_id, $this->Questions ) ) {
				$qdata = &$this->Questions[ $question_id ];
				$qdata->ProposalText[ $proposal_id ] = $row->proposal_text;
			}
		}
	}

	function getQuestionAnswer( $questionId, &$question ) {
		if ( $this->pid !== NULL ) {
			if ( array_key_exists( $questionId, $this->Questions ) ) {
				$quesData = $this->Questions[ $questionId ];
				if ( $quesData->IsCompatible( $question ) ) {
					return $quesData->getAnswer( $question );
				}
			}
		}
		return false;
	}

	function postQuestion( $questionId, &$question ) {
		if ( $this->pid !== NULL ) {
			$this->Questions[ $questionId ] = new qp_QuestionData( array(
				'from'=>'postdata',
				'type'=>$question->mType,
				'common_question'=>$question->mCommonQuestion,
				'categories'=>$question->mCategories,
				'category_spans'=>$question->mCategorySpans,
				'proposal_text'=>$question->mProposalText,
				'proposal_category_id'=>$question->mProposalCategoryId ) );
		}
	}

	function getState() {
		return $this->mCompletedPostData;
	}

	function stateIncomplete() {
		if ( $this->mCompletedPostData == 'NA' ) {
			$this->mCompletedPostData = 'incomplete';
		}
	}

	function stateError() {
		$this->mCompletedPostData = 'error';
	}

	function stateComplete() {
		# completed only when previous state was unavaibale; error state can't be completed
		if ( $this->mCompletedPostData == 'NA'  && count( $this->Questions ) > 0 ) {
			$this->mCompletedPostData = 'complete';
		}
	}

	function setLastUser( $username ) {
		if ( $this->pid !== NULL ) {
			$res = self::$db->select( 'qp_users','uid','name=' . self::$db->addQuotes( $username ), __METHOD__ );
			$row = self::$db->fetchObject( $res );
			if ( $row==false ) {
				self::$db->insert( 'qp_users', array( 'name'=>$username ), __METHOD__ . ':UpdateUser' );
				$this->last_uid = self::$db->insertId();
			} else {
				$this->last_uid = $row->uid;
			}
		}
//	todo: change to "insert ... on duplicate key update ..." when last_insert_id() bugs will be fixed
	}

	function getUserName( $uid ) {
		if ( $uid !== NULL) {
			$res = self::$db->select( 'qp_users','name','uid=' . self::$db->addQuotes( intval( $uid ) ), __METHOD__ );
			$row = self::$db->fetchObject( $res );
			if ( $row != false ) {
				return $row->name;
			}
		}
		return false;
	}

	private function loadPid() {
		$res = self::$db->select( 'qp_poll_desc',
			array( 'pid', 'fragment_id', 'dependance' ),
			'article_id=' . self::$db->addQuotes( $this->mArticleId ) . ' and ' .
			'poll_id=' . self::$db->addQuotes( $this->mPollId ),
			__METHOD__ );
		$row = self::$db->fetchObject( $res );
		if ( $row!=false ) {
			$this->pid = $row->pid;
			# some constructors don't supply the poll attributes, get the values from DB in such case
			if ( $this->mFragmentId === NULL ) {
				$this->mFragmentId = $row->fragment_id;
			}
			if ( $this->dependanceStr === NULL ) {
				$this->dependanceStr = $row->dependance;
			}
			$this->updatePollAttributes( $row );
		}
	}

	private function setPid() {
		$res = self::$db->select( 'qp_poll_desc',
			array( 'pid', 'fragment_id', 'dependance' ),
			'article_id=' . self::$db->addQuotes( $this->mArticleId ) . ' and ' .
			'poll_id=' . self::$db->addQuotes( $this->mPollId ) );
		$row = self::$db->fetchObject( $res );
		if ( $row==false ) {
			self::$db->insert( 'qp_poll_desc',
				array( 'article_id'=>$this->mArticleId, 'poll_id'=>$this->mPollId, 'fragment_id'=>$this->mFragmentId, 'dependance'=>$this->dependanceStr ),
				__METHOD__ . ':update poll' );
			$this->pid = self::$db->insertId();
		} else {
			$this->pid = $row->pid;
			$this->updatePollAttributes( $row );
		}
//	todo: change to "insert ... on duplicate key update ..." when last_insert_id() bugs will be fixed
	}

	private function updatePollAttributes( $row ) {
		if ( $this->mFragmentId != $row->fragment_id || $this->dependanceStr != $row->dependance ) {
			$res = self::$db->replace( 'qp_poll_desc',
				'',
				array( 'pid'=>$this->pid, 'article_id'=>$this->mArticleId, 'poll_id'=>$this->mPollId, 'fragment_id'=>$this->mFragmentId, 'dependance'=>$this->dependanceStr ),
				__METHOD__ . ':poll attributes update'
			);
		}
	}
	
	private function setQuestionDesc() {
		$insert = array();
		foreach ( $this->Questions as $qkey => &$ques ) {
			$insert[] = array( 'pid'=>$this->pid, 'question_id'=>$qkey, 'type'=>$ques->type, 'common_question'=>$ques->CommonQuestion );
			$ques->question_id = $qkey;
		}
		if ( count( $insert ) > 0 ) {
			self::$db->replace( 'qp_question_desc',
				'',
				$insert,
				__METHOD__ );
		}
	}

	private function setCategories() {
		$insert = Array();
		foreach ( $this->Questions as $qkey => &$ques ) {
			$ques->removeSpans();
			foreach ( $ques->Categories as $catkey => &$Cat ) {
				$insert[] = array( 'pid'=>$this->pid, 'question_id'=>$qkey, 'cat_id'=>$catkey, 'cat_name'=>$Cat["name"] );
			}
			$ques->restoreSpans();
		}
		if ( count( $insert ) > 0 ) {
			self::$db->replace( 'qp_question_categories',
				'',
				$insert,
				__METHOD__ );
		}
	}

	private function setProposals() {
		$insert = Array();
		foreach ( $this->Questions as $qkey => &$ques ) {
			foreach( $ques->ProposalText as $propkey => &$ptext ) {
				$insert[] = array( 'pid'=>$this->pid, 'question_id'=>$qkey, 'proposal_id'=>$propkey, 'proposal_text'=>$ptext );
			}
		}
		if ( count( $insert ) > 0 ) {
			self::$db->replace( 'qp_question_proposals',
				'',
				$insert,
				__METHOD__ );
		}
	}

	// warning: requires qp_PollStorage::last_uid to be set
	private function setAnswers() {
		$insert = Array();
		foreach ( $this->Questions as $qkey => &$ques ) {
			foreach( $ques->ProposalCategoryId as $propkey => &$prop_answers ) {
				foreach( $prop_answers as $catkey ) {
					$insert[] = array( 'uid'=>$this->last_uid, 'pid'=>$this->pid, 'question_id'=>$qkey, 'proposal_id'=>$propkey, 'cat_id'=>$catkey );
				}
			}
		}
		# TODO: delete votes of all users, when the POST question header is incompatible with question header in DB ?
		# delete previous vote to make sure previous header of this poll was not incompatible with current vote
		self::$db->delete( 'qp_question_answers',
			array( 'uid'=>$this->last_uid, 'pid'=>$this->pid ),
			__METHOD__ . ':delete previous answers of current user to the same poll'
		);
		# vote
		if ( count( $insert ) > 0 ) {
			self::$db->replace( 'qp_question_answers',
				'',
				$insert,
				__METHOD__ );
		}
	}

	# when the user votes and poll wasn't previousely voted yet, it also creates the poll in DB
	function setUserVote() {
		if ( $this->pid !==NULL &&
				$this->last_uid !== NULL &&
				$this->mCompletedPostData == "complete" &&
				is_array( $this->Questions ) && count( $this->Questions ) > 0 ) {
			$old_user_abort = ignore_user_abort( true );
			$this->setQuestionDesc();
			$this->setCategories();
			$this->setProposals();
			$this->setAnswers();
			ignore_user_abort( $old_user_abort );
			$this->voteDone = true;
		}
	}

}

/* parsing, checking ans visualisation of Question (UI input/output)
 */
class qp_Question {
	
	var $mQuestionType = "";
	var $mCategories = Array();
	var $mCategorySpans = Array();
	var $mProposalCategoryId = Array(); // POST user answers to the question's proposal
	var $mPrevProposalCategoryId = Array(); // GET user answers to the question's proposal
	var $alreadyVoted = false; // whether the selected user has already voted this question ?
	var $showResults = false; // whether to show the current stats to users
	// display layout
	var $proposalsFirst = false;
	var $transposed = false;
	var $spanType = 'colspan';
	var $categoriesStyle = '';
	var $proposalTextStyle = 'padding-left: 10px;';
	// statistics
	var $Percents = NULL;
	var $mRenderTable = Array();

	/**
	 * Constructor
	 * 
	 * @public
	 * @param  $beingCorrected			boolean.
	 * @param  $questionId				The Identifier of the question used to gernerate input names.
	 * @param  $parser					The wikitext parser.	 
	 */
	function __construct( $beingCorrected, $questionId, &$parser, $showResults = false ) {
		global $wgRequest;
		$this->mRequest = &$wgRequest;
		$this->mQuestionId = $questionId;
		$this->mBeingCorrected = $beingCorrected;
		$this->mParser = $parser;
		$this->mState = "";
		$this->mType = "multipleChoice";
		$this->showResults = $showResults;
		$this->mProposalPattern 	= '`^([+-]) ?(.*)`u';
		$this->mCategoryPattern 	= '`^\|(\n|[^\|].*\n)`u';
	}	

	/**
	 * Mutator of the question state
	 * 
	 * @protected
	 * @param  $pState
	 */
	function setState( $pState ) {
		if ( $this->mState != 'error' ) {
			$this->mState = $pState;
		}
	}

	/**
	 * Accessor of the question state.
	 * 
	 * @protected
	 */
	function getState() {
		return $this->mState;
	}

	/**
	 * Convert the question's header into HTML.
	 * 
	 * @param  $input				The poll header in QPoll syntax.
	 */
	function parseHeader($input) {
		$parametersPattern = '`\n\|([^\|].*)\s*$`u';
		$input = preg_replace_callback($parametersPattern, array($this, "parseParameters"), $input);
		$splitHeaderPattern = '`\n\|\|`u';
		$unparsedHeader = preg_split($splitHeaderPattern, $input);
		$output = $this->mParser->recursiveTagParse(trim($unparsedHeader[0])."\n");
		$this->mCommonQuestion = trim($output);
		return $output;
	}

	/**
	 * Determine the question's parameters.
	 * 
	 * @param  $matches				Elements matching $parametersPattern
	 * 								$matches[1] are the potential question parameters.
	 */
	function parseParameters($matches) {
		$paramkeys = array( 't[yi]p[eo]'=>NULL, 'layout'=>NULL );
		foreach( $paramkeys as $key=>&$val ) {
			preg_match( '`' . $key . '?="(.*?)"`u', $matches[1], $val );
		}
		$type = $paramkeys[ 't[yi]p[eo]' ];
		$layout = $paramkeys[ 'layout' ];
		if ( count( $layout ) > 0 ) {
			$this->transposed = strpos( $layout[1], 'transpose' ) !== false; 
			$this->proposalsFirst = strpos( $layout[1], 'proposals' ) !== false; 
		}
		# setup question layout parameters
		if ( $this->transposed ) {
			$this->spanType = 'rowspan';
			$this->categoriesStyle = 'text-align:left; vertical-align:middle; ';
			$this->signClass = array( 'first'=>'signt', 'middle'=>'signm', 'last'=>'signb' );
			$this->proposalTextStyle = 'text-align:center; padding-left: 5px; padding-right: 5px; ';
			$this->proposalTextStyle .= ( $this->proposalsFirst ) ? ' vertical-align: bottom;' : 'vertical-align:top;';
		} else {
			$this->spanType = 'colspan';
			$this->categoriesStyle = '';
			$this->signClass = array( 'first'=>'signl', 'middle'=>'signc', 'last'=>'signr' );
			$this->proposalTextStyle = 'vertical-align:middle; ';
			$this->proposalTextStyle .= ( $this->proposalsFirst ) ? 'padding-right: 10px;' : 'padding-left: 10px;';
		}
		if( count( $type ) > 0 ) {
			# List of all object type code and the correspondant question type.
			switch ( $type[1] ) {
			case 'unique()':
				$this->mQuestionType = "unique";
				$this->mType = "singleChoicePoll";
				break;
			case '()':
				$this->mType = "singleChoicePoll";
				break;
			case '[]':
				$this->mType = "multipleChoicePoll";
				break;
			}
		} else {
			$this->setState( "error" );
			$this->mType = "error";
		}
		return;
	}

	/**
	 * Transmit a single choice object header to the basic type parser.
	 * 
	 * setup object lines raws
	 *
	 * @param  $input				A question object in QPoll syntax.
	 */
	function singleChoicePollParseObjectHeader( $input ) {
		$this->pollParseObjectHeader( $input, "radio" );
	}

	function multipleChoicePollParseObjectHeader( $input ) {
		$this->pollParseObjectHeader( $input, "checkbox" );
	}

	function pollParseObjectHeader( $input, $inputType ) {
		$this->raws = preg_split('`\n`su', $input, -1, PREG_SPLIT_NO_EMPTY);
		if ( array_key_exists( 1, $this->raws ) ) {
			$categorySpans = preg_match($this->mCategoryPattern, $this->raws[1]."\n", $matches);
		}
		if ( !$categorySpans && array_key_exists( 0, $this->raws ) ) {
			preg_match($this->mCategoryPattern, $this->raws[0]."\n", $matches);
		}
		# parse the header - spans and categories
		$catString = isset( $matches[1] ) ? $matches[1] : '';
		$catRow = $this->parseCategories( $catString );
		if ( $categorySpans ) {
			$spansRow = $this->parseCategorySpans($this->raws[0]);
			# if there are multiple spans, "turn on" borders for span and category cells
			if ( count( $this->mCategorySpans ) > 1 ) {
				$this->categoriesStyle .= 'border:1px solid gray;';
			}
			if ( $this->categoriesStyle != '' ) {
				qp_Renderer::applyAttrsToRow( $spansRow, array( 'style'=>$this->categoriesStyle ) );
			}
			$this->addRow( $spansRow, array( 'class'=>'spans'), 'th', array( 'count'=>$this->spanType, 'name'=>0 ) );
		}
		if ( $this->categoriesStyle != '' ) {
			qp_Renderer::applyAttrsToRow( $catRow, array( 'style'=>$this->categoriesStyle ) );
		}
		$this->addRow( $catRow, array( 'class'=>'categories'), 'th', array( 'name'=>0 ) );
	}

	/**
	 * Transmit a single choice object body to the basic type parser.
	 * 
	 * @param  $input				A question object in QPoll syntax.
	 * 
	 * @return $output				A question object in HTML.
	 */
	function singleChoicePollParseObject() {
		$this->pollParseObject( "radio" );
		return qp_Renderer::renderHTMLobject( $this->mRenderTable );
	}

	function multipleChoicePollParseObject() {
		$this->pollParseObject( "checkbox" );
		return qp_Renderer::renderHTMLobject( $this->mRenderTable );
	}

	function pollParseObject( $inputType ) {
		# Parameters used in some special cases.
		$this->mProposalPattern = '`^[^\|\!].*`u';
		$proposalId = -1;
		foreach( $this->raws as $raw ) {
			$text 			= NULL;
			$row = Array();
			if ( preg_match( $this->mProposalPattern, $raw, $matches ) ) {
				$proposalId++;
				$rawClass = "proposal";
				# Insulate the proposal signes.
				$text = array_pop( $matches );
				$this->mProposalText[ $proposalId ] = trim( $text );
				# Determine a type ID, according to the questionType and the number of signes.
				$typeId  = substr( $this->mType, 0, 1 );
				$prevSpanId = -1;
				foreach ( $this->mCategories as $catId => $catDesc ) {
					$row[ $catId ] = Array();
					$inp = Array( "__tag"=>"input" );
					$signClass = "sign";
					# Determine the input's name and value.
					switch( $typeId ) {
					case "m":
						$name = 'q' . $this->mQuestionId . 'p' . $proposalId . 's' . $catId;
						$value = 's' . $catId;
						break;
					case "s":
						$name = 'q' . $this->mQuestionId . 'p' . $proposalId;
						$value = 's' . $catId;
						# category spans have sense only with single choice proposals
						if ( array_key_exists( "spanId", $catDesc ) ) {
							$spanId = $catDesc[ "spanId" ];
							$name .= 's' . $spanId;
							# there can't be more category spans than the categories
							if ( $this->mCategorySpans[ $spanId ]["count"] > count( $this->mCategories ) ) {
								$text = $this->parseErrorMessage( wfMsg( "qp_error_too_many_spans" ), "error" ) . $text;
								$rawClass = "proposalerror";
							}
							if ($prevSpanId != $spanId) {
								# begin of new span
								$span_was_checked = false;
								$prevSpanId = $spanId;
								$spanCount = $this->mCategorySpans[ $spanId ]["count"];
								if ( $spanCount < 2 ) {
									$text = $this->parseErrorMessage( wfMsg( "qp_error_too_few_spans" ), "error" ) . $text;
									$row[ $catId ][ "style" ] = QP_CSS_ERROR_STYLE;
								}
								$drawSpan = $spanCount != 1 && $spanCount != count( $this->mCategories );
								# hightlight only spans of count != 1 and count != count(categories)
								if ( $drawSpan ) {
									$signClass = $this->signClass[ 'first' ];
								}
							} else {
								$spanCount--;
								if ( $drawSpan ) {
									$signClass = ( $spanCount > 1 ) ? $this->signClass[ 'middle' ] : $this->signClass[ 'last' ];
								}
							}
							if ($spanCount <= 1 ) {
								# end of new span
								if ( $this->mBeingCorrected && 
										!$span_was_checked &&
										$this->mRequest->getVal($name) != $value ) {
									# the span (a part of proposal) was submitted but unanswered
									$text = $this->parseErrorMessage( wfMsg( "qp_error_unanswered_span" ), "NA" ) . $text;
									# highlight current span to indicate an error
									for ( $i = $catId, $j = $this->mCategorySpans[ $spanId ]["count"]; $j > 0; $i--, $j-- ) {
										$row[$i][ "style" ] = QP_CSS_ERROR_STYLE;
									}
									$rawClass = "proposalerror";
								}
							}
						}
						break;
					}
					# Determine if the input had to be checked.
					if ( $this->mBeingCorrected && $this->mRequest->getVal($name) == $value ) {
						$inp[ 'checked' ] = 'checked';
					}
					if ( $this->alreadyVoted &&
							array_key_exists( $proposalId, $this->mPrevProposalCategoryId ) &&
							in_array( $catId, $this->mPrevProposalCategoryId[ $proposalId ] ) ) {
						$inp[ 'checked' ] = 'checked';
					}
					if ( array_key_exists( 'checked', $inp ) ) {
						if ( $this->mQuestionType == "unique" ) {
							if ( $this->mBeingCorrected && !$this->isUniqueProposalCategoryId( $proposalId, $catId ) ) {
								$text = $this->parseErrorMessage( wfMsg( "qp_error_non_unique_choice" ), "NA" ) . $text;
								$rawClass = "proposalerror";
								unset( $inp[ 'checked' ] );
								$row[ $catId ][ "style" ] = QP_CSS_ERROR_STYLE;
							}
						} else {
							$span_was_checked = true;
						}
					}
					if ( array_key_exists( 'checked', $inp ) ) {
						$this->mProposalCategoryId[ $proposalId ][]=$catId;
					}
					$row[ $catId ][ "class" ] = $signClass;
					if ( $this->mQuestionType == "unique" ) {
						# unique question category proposal ID for javascript
						$inp[ "id" ] = 'uq' . $this->mQuestionId . 'c' . $catId . 'p' . $proposalId;
						# If type="unique()" question has more proposals than categories, such question is impossible to complete
						if ( count( $this->mProposalText ) > count( $this->mCategories ) ) {
							# if there was no previous errors, hightlight the whole row
							if ( $this->getState() == "" ) {
								foreach( $row as &$cell ) {
									$cell[ "style" ] = QP_CSS_ERROR_STYLE;
								}
							}
							$text = $this->parseErrorMessage( wfMsg( "qp_error_unique" ), "error" ) . $text;
							$rawClass = "proposalerror";
						}
					}
					$inp[ "class" ] = "check";
					$inp[ "type" ] = $inputType;
					$inp[ "name" ] = $name;
					$inp[ "value" ] = $value;
					if ( $this->showResults &&
							is_array( $this->Percents ) &&
							array_key_exists( $proposalId, $this->Percents ) ) {
						# there ars some stat in row (not necessarily all cells, because size of poll table changes dynamically)
						$cell_stat = "&nbsp;";
						if ( is_array( $this->Percents[ $proposalId ] ) &&
								array_key_exists( $catId, $this->Percents[ $proposalId ] ) ) {
							# there is a stat in cell
							$cell_stat = intval( round( 100 * $this->Percents[ $proposalId ][ $catId ] ) ) . '%';
						}
						$row[ $catId ][ 0 ] = array( array( '__tag'=>'div', 0=>$inp ), array( '__tag'=>'div', 'class'=>'stats', 0=>$cell_stat ) );
					} else {
						$row[ $catId ][ 0 ] = $inp;
					}
				}
				# If the proposal text is empty, the question has a syntax error.
				if( trim( $text ) == "" ) {
					$text = $this->parseErrorMessage( wfMsg( "qp_error_proposal_text_empty" ), "error" );
					foreach( $row as &$cell ) {
						$cell[ "style" ] = QP_CSS_ERROR_STYLE;
					}
					$rawClass = "proposalerror";
				}
				# If the proposal was submitted but unanswered
				if ( $this->mBeingCorrected && !array_key_exists( $proposalId, $this->mProposalCategoryId ) ) {
					# if there was no previous errors, hightlight the whole row
					if ( $this->getState() == "" ) {
						foreach( $row as &$cell ) {
							$cell[ "style" ] = QP_CSS_ERROR_STYLE;
						}
					}
					$text = $this->parseErrorMessage( wfMsg( "qp_error_no_answer" ), "NA" ) . $text;
					$rawClass = "proposalerror";
				}
			}
			if($text) {
				$text = array( '__tag'=>'td', '__end'=>"\n", 'class'=>'proposaltext', 'style'=>$this->proposalTextStyle, 0=>$this->mParser->recursiveTagParse( $text ) );
				if ( $this->proposalsFirst ) {
					# first element is proposaltext
					array_unshift( $row, $text );
				} else {
					# last element is proposaltext
					$row[] = $text;
				}
				$this->addRow( $row, array( 'class'=>$rawClass ), 'td' );
			}
		}
	}

	function addRow( $row, $rowattrs = "", $celltag = "td", $attribute_maps = NULL ) {
		if ( $this->transposed ) {
			qp_Renderer::addColumn( $this->mRenderTable, $row, $rowattrs, $celltag, $attribute_maps );
		} else {
			qp_Renderer::addRow( $this->mRenderTable, $row, $rowattrs, $celltag, $attribute_maps );
		}
	}

	/**
	 * Determine the object's parameters and convert a list of categories from QPoll syntax to HTML.
	 * 
	 * @param  $input			The various categories.
	 */
	function parseCategories($input) {
		$row = Array();
		if ( $this->proposalsFirst ) {
			// add empty <th> at the begin of row to "compensate" proposal text
			$row[] = array( '__tag'=>'td', 0=>"", 'style'=>'border:none;', '__end'=>"\n" );
		}
		$categories = explode('|', $input);
		# Less than two categories is a syntax error.
		if( !array_key_exists( 1, $categories ) ) {
			$categories[0] .= $this->parseErrorMessage( wfMsg( "qp_error_too_few_categories" ), "error" );
		}
		foreach( $categories as $catkey => $category ) {
			$category = trim( $category );
			# If a category name is empty, the question has a syntax error.
			if( $category == "") {
				$category = $this->parseErrorMessage( wfMsg( "qp_error_category_name_empty" ), "error" );
			}
			$currcateg = $this->mParser->recursiveTagParse( $category );
			$row[] = $currcateg;
			$this->mCategories[ $catkey ]["name"] = $currcateg;
		}
		# cut unused categories rows which are presented in DB but were removed from article
		$this->mCategories = array_slice( $this->mCategories, 0, count( $categories ) );
		if ( !$this->proposalsFirst ) {
			// add empty <th> at the end of row to "compensate" proposal text
			$row[] = array( '__tag'=>'td', 0=>"", 'style'=>'border:none;', '__end'=>"\n" );
		}
		return $row;
	}
	
	/**
	 * Determine the object's parameters and convert a list of category spans from QPoll syntax to HTML.
	 * 
	 * @param  $input			The various headers of categories.
	 */
	# warning: parseCategorySpans() should be called after parseCategories()
	function parseCategorySpans($input) {
		$row = Array();
		if ( $this->mType == "singleChoicePoll" ) {
			# real category spans have sense only for radiobuttons
			preg_match_all('`([\|\!])([^\|\!]*)`u', $input, $spans);
			array_shift( $spans );
			# Less than one span is a syntax error.
			if(!array_key_exists(0, $spans)) {
				return $this->parseErrorMessage( wfMsg( "qp_error_too_few_spans" ), "error" );
			}
			# fill undefined spans with the last span value
			$SpanCategDelta = count( $this->mCategories ) - count( $spans[0] );
			$lastDefinedSpanKey = array_pop( array_diff( array_keys( $spans[1] ), array_keys( $spans[1], "", true ) ) );
			if ($lastDefinedSpanKey !== NULL) {
				if ( $SpanCategDelta > 0 ) {
					# increase the length of last defined span value to match total lenth of categories
					$lastSpanType = $spans[0][$lastDefinedSpanKey];
					$spans[0] = array_merge( array_slice( $spans[0], 0, $lastDefinedSpanKey ),
						array_fill( 0, $SpanCategDelta, $lastSpanType ),
						array_slice( $spans[0], $lastDefinedSpanKey ) );
					$spans[1] = array_merge( array_slice( $spans[1], 0, $lastDefinedSpanKey ),
						array_fill( 0, $SpanCategDelta, "" ),
						array_slice( $spans[1], $lastDefinedSpanKey ) );
				} elseif ( $SpanCategDelta < 0 ) {
					# cut unused but defined extra spans
					$spans[0] = array_slice( $spans[0], count( $this->mCategories ), -$SpanCategDelta );
					$spans[1] = array_slice( $spans[1], count( $this->mCategories ), -$SpanCategDelta );
				}
			} else {
				# no valid category spans are defined
				return $this->parseErrorMessage( wfMsg( "qp_error_too_few_spans" ), "error" ); 
			}
			# populate mCategorySpans and row
			if ( $this->proposalsFirst ) {
				// add empty <th> at the begin of row to "compensate" proposal text
				$row[] = array( '__tag'=>'td', 0=>"", 'style'=>'border:none;', '__end'=>"\n" );
			}
			$colspanBase = ( $lastDefinedSpanKey == 0 ) ? 1 : 0;
			$colspan = 1;
			$categorySpanId = 0;
			foreach($spans[0] as $spanKey => $spanType) {
				$spanCategory = trim( $spans[1][$spanKey] );
				if ( $spanCategory=="" ) {
					$colspan++;
				} else {
					$row[] = array( "count"=>$colspan + $colspanBase, 0=>$this->mParser->recursiveTagParse( $spanCategory ) );
					if ( $spanType == "|") { // "!" is a comment header, not a real category span
						$this->mCategorySpans[ $categorySpanId ]["name"] = $spanCategory;
						$this->mCategorySpans[ $categorySpanId ]["count"] = $colspan;
						for ( $i = $spanKey;
							$i >=0 && array_key_exists( $i, $this->mCategories ) && !array_key_exists( "spanId", $this->mCategories[ $i ] );
							$i-- ) {
							$this->mCategories[$i]["spanId"] = $categorySpanId;
						}
						$categorySpanId++;
					}
					$colspan = 1;
				}
			}
			if ( !$this->proposalsFirst ) {
				// add empty <th> at the end of row to "compensate" proposal text
				$row[] = array( '__tag'=>'td', 0=>"", 'style'=>'border:none;', '__end'=>"\n" );
			}
		}
		return $row;
	}

	function isUniqueProposalCategoryId( $proposalId, $catId ) {
		foreach( $this->mProposalCategoryId as $proposalCategoryId ) {
			if ( in_array( $catId, $proposalCategoryId ) ) {
				return false;
			}
		}
		return true;
	}

	function parseErrorMessage( $error_msg, $state ) {
		$prev_state = $this->getState();
		$this->setState( $state );
		# return the message only for the first error occured
		return ( $prev_state == "" ) ? "<span class=\"proposalerror\" title=\"$error_msg\">???</span> " : "";
	}
}

/* render output data */
class qp_Renderer {
	// the stucture of $tag is like this:
	// array( "__tag"=>"td", "class"=>"myclass", 0=>"text before li", 1=>array( "__tag"=>"li", 0=>"text inside li" ), 2=>"text after li" )
	// both tagged and tagless lists are supported
	static function renderHTMLobject( &$tag ) {
		$tag_open = "";
		$tag_close = "";
		$tag_val = NULL;
		if ( is_array( $tag ) ) {
			ksort( $tag );
			if ( array_key_exists( '__tag', $tag ) ) {
				# list inside of tag
				$tag_open .= "<" . $tag[ '__tag' ];
				foreach( $tag as $attr_key=>&$attr_val ) {
					if ( is_int( $attr_key ) ) {
						if ( $tag_val === NULL )
							$tag_val = "";
						if ( is_array( $attr_val ) ) {
							# recursive tags
							$tag_val .= self::renderHTMLobject( $attr_val );
						} else {
							# text
							$tag_val .= $attr_val;
						}
					} else {
						# string keys are for tag attributes
						if ( substr( $attr_key, 0, 2 ) != "__" ) {
							# include only non-reserved attributes
							$tag_open .= " $attr_key=\"" . $attr_val . "\"";
						}
					}
				}
				if ( $tag_val !== NULL ) {
					$tag_open .= ">";
					$tag_close .= "</" . $tag[ '__tag' ] . ">";
				} else {
					$tag_open .= " />";
				}
				if ( array_key_exists( '__end', $tag ) ) {
					$tag_close .= $tag[ '__end' ];
				}
			} else {
				# tagless list
				$tag_val = "";
				foreach( $tag as $attr_key=>&$attr_val ) {
					if ( is_int( $attr_key ) ) {
						if ( is_array( $attr_val ) ) {
							# recursive tags
							$tag_val .= self::renderHTMLobject( $attr_val );
						} else {
							# text
							$tag_val .= $attr_val;
						}
					} else {
						ob_start();
						var_dump($tag);
						$tagdump=ob_get_contents();
						ob_end_clean();
						$tag_val = "invalid argument: tagless list cannot have tag attribute values in key=$attr_key, $tagdump";
					}
				}
			}
		} else {
			# just a text
			$tag_val = $tag;
		}
		return $tag_open . $tag_val . $tag_close;
	}

	# creates one "htmlobject" row of the table
	# elements of $row can be either a string/number value of cell or an array( "count"=>colspannum, "attribute"=>value, 0=>html_inside_tag )
	# attribute maps can be like this: ("name"=>0, "count"=>colspan" )
	static function newRow( $row, $rowattrs = "", $celltag = "td", $attribute_maps = NULL ) {
		$result = "";
		if ( count( $row ) > 0 ) {
			foreach ( $row as &$cell ) {
				if ( !is_array( $cell ) ) {
					$cell = array( 0=>$cell );
				}
				$cell[ '__tag' ] = $celltag;
				$cell[ '__end' ] = "\n";
				if ( is_array( $attribute_maps ) ) {
					# converts ("count"=>3) to ("colspan"=>3) in table headers - don't use frequently
					foreach ( $attribute_maps as $key=>$val ) {
						if ( array_key_exists( $key, $cell ) ) {
							$cell[ $val ] = $cell[ $key ];
							unset( $cell[ $key ] );
						}
					}
				}
			}
			$result = array( '__tag'=>'tr', 0=>$row, '__end'=>"\n" );
			if ( is_array( $rowattrs ) ) {
				$result = array_merge( $rowattrs, $result );
			} elseif ( $rowattrs !== "" )  {
				$result[0][] = __METHOD__ . ':invalid rowattrs supplied';
			}
		}
		return $result;
	}

	# add row to the table
	static function addRow( &$table, $row, $rowattrs = "", $celltag = "td", $attribute_maps = NULL ) {
		$table[] = self::newRow( $row, $rowattrs, $celltag, $attribute_maps );
	}

	# add column to the table
	static function addColumn( &$table, $column, $rowattrs = "", $celltag = "td", $attribute_maps = NULL ) {
		if ( count( $column ) > 0 ) {
			$row = 0;
			foreach ( $column as &$cell ) {
				if ( !is_array( $cell ) ) {
					$cell = array( 0=>$cell );
				}
				$cell[ '__tag' ] = $celltag;
				$cell[ '__end' ] = "\n";
				if ( is_array( $attribute_maps ) ) {
					# converts ("count"=>3) to ("rowspan"=>3) in table headers - don't use frequently
					foreach ( $attribute_maps as $key=>$val ) {
						if ( array_key_exists( $key, $cell ) ) {
							$cell[ $val ] = $cell[ $key ];
							unset( $cell[ $key ] );
						}
					}
				}
				if ( is_array( $rowattrs ) ) {
					$cell = array_merge( $rowattrs, $cell );
				} elseif ( $rowattrs !== "" ) {
					$cell[ 0 ] = __METHOD__ . ':invalid rowattrs supplied';
				}
				if ( !array_key_exists( $row, $table ) ) {
					$table[ $row ] = array( '__tag'=>'tr', '__end'=>"\n" );
				}
				$table[ $row ][] = $cell;
				if ( array_key_exists( 'rowspan', $cell ) ) {
					$row += intval( $cell[ 'rowspan' ] );
				} else {
					$row++;
				}
			}
			$result = array( '__tag'=>'tr', 0=>$column, '__end'=>"\n" );
		}
	}

	static function displayRow( $row, $rowattrs = "", $celltag = "td", $attribute_maps = NULL ) {
		return self::renderHTMLobject( self::newRow( $row, $rowattrs, $celltag, $attribute_maps ) );
	}

	// use newRow() or addColumn() to add resulting row/column to the table
	// if you want to use the resulting row with renderHTMLobject(), don't forget to apply attrs=array('__tag'=>'td')
	static function applyAttrsToRow( &$row, $attrs ) {
		if ( is_array( $attrs ) && count( $attrs > 0 ) ) {
			foreach( $row as &$cell ) {
				if ( !is_array( $cell ) ) {
					$cell = array_merge( $attrs, array( $cell ) );
				} else { 
					foreach( $attrs as $attr_key=>$attr_val ) {
						if ( !array_key_exists( $attr_key, $cell ) ) {
							$cell[ $attr_key ] = $attr_val;
						}
					}
				}
			}
		}
	}
}

class qp_FunctionsHook {

	function qpuserchoice( &$parser, $frame, $args ) {
		$error_message = 'qp_func_no_such_poll';
		$error_polladdr = 'noname';
		$error_qid = '';
		$error_proposal_id = '';
		if ( isset( $args[ 0 ] ) ) {
			qp_Poll::loadMessages();
			# build dependance poll object
			$dependanceStr = trim( $frame->expand( $args[ 0 ] ) );
			$error_polladdr = $dependanceStr;
			$depSplit = qp_Poll::splitDependanceString( $dependanceStr );
			if ( is_array( $depSplit ) ) {
				$dependanceStr = qp_Poll::buildDependanceString( $depSplit );
				list( $depTitleStr, $depPollId ) = $depSplit;
				$depTitle = Title::newFromURL( $depTitleStr );
				if ( $depTitle !== NULL ) {
					$depArticleId = intval( $depTitle->getArticleID() );
					if ( $depArticleId > 0 ) {
						$depPollStore = new qp_PollStore( array(
							'from'=>'poll_get',
							'title'=>$depTitle,
							'poll_id'=>$depPollId ) );
						if ( $depPollStore->pid !== NULL ) {
							$error_message = 'qp_func_missing_question_id';
							if ( isset( $args[ 1 ] ) ) {
								$question_id = trim( $frame->expand( $args[ 1 ] ) );
								$error_qid = $question_id;
								$error_message = 'qp_func_invalid_question_id';
								if ( is_numeric( $question_id ) ) {
									$question_id = intval( $question_id );
									$depPollStore->loadQuestions();
									$depPollStore->setLastUser( qp_Poll::currentUserName() );
									$depPollStore->loadUserVote();
									$error_message = 'qp_func_missing_question_id';
									if ( array_key_exists( $question_id, $depPollStore->Questions ) ) {
										$ques = &$depPollStore->Questions[ $question_id ];
										$error_message = 'qp_func_missing_proposal_id';
										if ( isset( $args[ 2 ] ) ) {
											# get poll's proposal choice
											$proposal_id = trim( $frame->expand( $args[ 2 ] ) );
											$error_proposal_id = $proposal_id;
											$error_message = 'qp_func_invalid_proposal_id';
											if ( is_numeric( $proposal_id ) ) {
												$defaultProposalText = isset( $args[ 3 ] ) ? trim( $frame->expand( $args[ 3 ] ) ) : '';
												$proposal_id = intval( $proposal_id );
												$error_message = 'qp_func_missing_proposal_id';
												if ( array_key_exists( $proposal_id, $ques->ProposalText ) ) {
													$result = '';
													if ( array_key_exists( $proposal_id, $ques->ProposalCategoryId ) ) {
														foreach( $ques->ProposalCategoryId[ $proposal_id ] as $cat_id ) {
															if ( $result != '' ) {
																$result .= '~';
															}
															$cat_name = isset( $args[ $cat_id + 4 ] ) ? trim( $frame->expand( $args[ $cat_id + 4 ] ) ) : '';
															if ( $cat_name != '' ) {
																$result .= $cat_name;
															} else {
																if ( $defaultProposalText != '' ) {
																	$result .= $defaultProposalText;
																} else {
																	$result .= $ques->Categories[ $cat_id ][ "name" ];
																}
															}
														}
													}
													return $result;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return array( 0=>'qpuserchoice: ' . wfMsgHtml( $error_message, $error_polladdr, $error_qid, $error_proposal_id ), 'isHTML'=>true );
	}

	static function ParserFunctionsWords( $lang ) {
		$words = array();
		$words[ 'en' ] = array( 'qpuserchoice'=>array( 0, 'qpuserchoice' ) );
		# English is used as a fallback, and the English synonyms are
		# used if a translation has not been provided for a given word
		return ( $lang == 'en' || !array_key_exists( $lang, $words ) )
			? $words[ 'en' ]
			: array_merge( $words[ 'en' ], $words[ $lang ] );
	}

	static function languageGetMagic( &$magicWords, $langCode ) {
		foreach( qp_FunctionsHook::ParserFunctionsWords( $langCode ) as $word => $trans )
			$magicWords [$word ] = $trans;
		return true;
	}

}
